# build_static_baseline.R
#
# builds a baseline file that gets sage to run as a static model.
# the output is stored in data/baseline_static.gms by default



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# directory where data files will be stored
data_dir = file.path("data")

# gams input file to create
output.file = file.path(data_dir,"baseline_static.gms")

# if the sage years are not defined, specify a single year - the value doesn't
# really matter
if (!exists("sage.years"))
  sage.years = 2016



# ------------------------------------------------------------------------------
# write model years to the baseline
# ------------------------------------------------------------------------------

# write the set defining the model's years
fc = file(output.file,"w")
writeLines("sets",con=fc)
writeLines(paste0("  t                     time periods            / ",
                  sage.years[1]," /;"),
           con=fc)
writeLines("",con=fc)



# ------------------------------------------------------------------------------
# "zero" out any baseline growth
# ------------------------------------------------------------------------------

writeLines("l_prod(t,s)         = 1;",con=fc)
writeLines("l_prod_agg(t)       = 1;",con=fc)
writeLines("l_force(t)          = 1;",con=fc)
writeLines("cd_ene_growth(t,s)  = 1;",con=fc)
writeLines("ene_growth(t,ss,s)  = 1;",con=fc)
writeLines("col_ele_growth(t)   = 1;",con=fc)
writeLines("curactbal(t)        = 1;",con=fc)
writeLines("deficit(t)          = 1;",con=fc)
writeLines("gint(t)             = 1;",con=fc)
writeLines("gint_row(t)         = 1;",con=fc)
writeLines("gov_exp(t)          = 1;",con=fc)
writeLines("transfers(t)        = 1;",con=fc)
writeLines("oth_adj(t)          = 1;",con=fc)
writeLines("inc_row(t)          = 1;",con=fc)
writeLines("tran_row(t)         = 1;",con=fc)



# ------------------------------------------------------------------------------
# redefine rates
# ------------------------------------------------------------------------------

# if this is a static model run then redefine the rate parameters - in the
# static run these parameters essentially lose their meaning so these values are
# just to ensure the solution represents a static run and the values should not
# be interpreted with the parameters normal meaning for the dynamic model
delta = 1.0
rbar = 0.0

# macroeconomic variables
comments = "rate - depreciation rate"
write.gams.variable(fc,"delta",delta,comments=comments,write.zeros=TRUE)
comments = "rate - benchmark interest rate"
write.gams.variable(fc,"rbar",rbar,comments=comments,write.zeros=TRUE)

# close the connection to the output file
close(fc)
