# get_additional_economic_Data.R
#
# downloads additional economic account data need by build/build_benchmark.gms
# gams input file is stored in build/data/additional_economic_data.gms
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# directory where data files will be stored
data_dir = file.path("build","data")

# gams input file to create
benchmark.data.file = file.path(data_dir,"additional_benchmark_data.gms")



# ------------------------------------------------------------------------------
# get additional economic data needed for the benchmark
# ------------------------------------------------------------------------------

# the benchmark year is the first year in the model
benchmark_year = sage.years[1]

# government deficit
deficit0 = -get.bea.variable("T30100",benchmark_year,row=43)[1]/1000

# interest payments domestically
gint0 = get.bea.variable("T30100",benchmark_year,row=28)[1]/1000

# interest payments to rest of world
gint_row0 = get.bea.variable("T30100",benchmark_year,row=29)[1]/1000

# federal plus state and local debt in the benchmark year
# https://fred.stlouisfed.org/series/GFDEBTN#0
# https://fred.stlouisfed.org/series/SLGSDODNS#0
debt0 = 19549.2005+3125.39525

# current account balance
curactbal0 = get.bea.variable("T40100",benchmark_year,row=35)[1]/1000

# net income receipts from the rest of the world
inc_row0 = (get.bea.variable("T40100",benchmark_year,row=7)[1]-
            get.bea.variable("T40100",benchmark_year,row=23)[1])/1000+
           gint_row0

# net tax and transfer receipts from the rest of the world
tran_row0 = (get.bea.variable("T40100",benchmark_year,row=13)[1]-
             get.bea.variable("T40100",benchmark_year,row=29)[1])/1000

# write to the gams input file
write.gams.variable(benchmark.data.file,"curactbal0",curactbal0,
                    comments="benchmark - current account balance")
write.gams.variable(benchmark.data.file,"inc_row0",inc_row0,append=TRUE,
                    comments="benchmark - net income receipts from rest of world")
write.gams.variable(benchmark.data.file,"tran_row0",tran_row0,append=TRUE,
                    comments="benchmark - net tax and transfer receipts from rest of world")
write.gams.variable(benchmark.data.file,"deficit0",deficit0,append=TRUE,
                    comments="benchmark - government deficit")
write.gams.variable(benchmark.data.file,"gint0",gint0,append=TRUE,
                    comments="benchmark - government interest payments domestically")
write.gams.variable(benchmark.data.file,"gint_row0",gint_row0,append=TRUE,
                    comments="benchmark - government interest payments to rest of world")
write.gams.variable(benchmark.data.file,"debt0",debt0,append=TRUE,
                    comments="benchmark - government debt")
