* simulates the income elasticities of consumption
*
* run as an "epilogue" using:
*   gams model/sage.gms --epilogue=examples/income_elasticity.gms
*
* see examples/income_elasticity.R for a run script with results plot
*



* ------------------------------------------------------------------------------
* options
* ------------------------------------------------------------------------------

* set the path separator based on the operating system
$iftheni %system.filesys% == UNIX $set sep "/"
$else $set sep "\"
$endif

* set default name for output file if needed
$if not set ie_output_file $set ie_output_file "output%sep%income_elasticity_results.csv"



* ------------------------------------------------------------------------------
* save the baseline results
* ------------------------------------------------------------------------------

parameters
  cds_base(t,r,s,h)    baseline consumption
  m_base(t,r,h)        baseline income;

* save baseline consumption
cds_base(t,r,s,h) = cd.l(t,r,s,h)*cd_base(t,r,s,h);

* save total baseline consumption expenditures
m_base(t,r,h) = sum(s, pa.l(t,r,s)*cds_base(t,r,s,h));



* ------------------------------------------------------------------------------
* define mcp problem for the income elasticitiy
* ------------------------------------------------------------------------------

* fix the prices at the policy case levels
pa.fx(t,r,s)      = pa.l(t,r,s);
pk.fx(t,r)        = pk.l(t,r);
pr.fx(t,r)        = pr.l(t,r);
pl.fx(t,r)        = pl.l(t,r);
pfx.fx(t)         = pfx.l(t);
pkt.fx(r)         = pkt.l(r);
kt.fx(r)          = kt.l(r);
pr_ex.fx(t,r,s)   = pr_ex.l(t,r,s);
incadj.fx(t)      = incadj.l(t);
pres.fx(t,r,s)    = pres.l(t,r,s);
res_ex.fx(t,r,s)  = res_ex.l(t,r,s);
res.fx(t,r,s)     = res.l(t,r,s);
cpi.fx(t)         = cpi.l(t);
pr_ex_agg.fx(t,r) = pr_ex_agg.l(t,r);
pkh.fx(t)         = pkh.l(t);
prh.fx(t)         = prh.l(t);
prh_ex.fx(t)      = prh_ex.l(t);
pkht.fx           = pkht.l;
kht.fx(r,h)       = kht.l(r,h);

* define the model for computing the uncompensated elasticity
model hh_opt /
  zp_c.dc
  zp_cl.pcl

  hde_dc.pc
  hde_c.c
  hde_cd.cd
  hde_dcd.dcd
  hde_scd.scd
  hde_leis.leis

  mc_time.l
  mc_invh.invh

  bc_hh.lambda

  foc_cl.cl
  foc_lambda.kh

  uce_c.uc_c
  uce_cl.uc_cl

  aux_k_ex.k_ex
  aux_kh_ex.kh_ex
  aux_w.w
/;

* iteration limit
hh_opt.iterlim = 1000000;

* use model scaling
hh_opt.scaleopt = 1;

* turn off solution output
hh_opt.solprint = 0;



* ------------------------------------------------------------------------------
* calculate the income elasticity
* ------------------------------------------------------------------------------

parameters
  income_elasticity(t,r,s,h)  income elasticity;

* increase income by a small percentage
inc_extra.fx(t,r,h) = income(t,r,h)*.0001;

* solve the mcp problem
solve hh_opt using mcp;

* calculate the income elasticity
* calculated with respect to the change in total expenditures on commodities to
* match the calibration procedure and the target elasticities
income_elasticity(t,r,s,h)$(cds_base(t,r,s,h))
  = ((cd.l(t,r,s,h)*cd_base(t,r,s,h))
      /cds_base(t,r,s,h)-1)
    /(sum(ss, pa.l(t,r,ss)*(cd.l(t,r,ss,h)*cd_base(t,r,ss,h)))
      /m_base(t,r,h)-1);

* output csv results file
file ie_results /"%ie_output_file%"/;
ie_results.pc = 5;
ie_results.nd = 6;
put ie_results;
put "time", "region", "sector", "household", "value" /;
loop((t,r,s,h), put t.tl, r.tl, s.tl, h.tl, income_elasticity(t,r,s,h) / );
putclose;
