$title macro_sam



* ------------------------------------------------------------------------------
* set options
* ------------------------------------------------------------------------------

* set the path separator based on the operating system
$iftheni %system.filesys% == UNIX $set sep "/"
$else $set sep "\"
$endif

* set default benchmark file
$if not set benchmark_file $set benchmark_file "data%sep%default_aggregation.gdx"



* ------------------------------------------------------------------------------
* load model sets
* ------------------------------------------------------------------------------

* basic model sets
sets
  s                     sectors
  r                     regions
  h                     households
  trd                   trade markets
  f                     primary factors
  t                     time periods;

* set aliases
alias
  (s,ss,sss), (h,hh), (r,rr);

$gdxin %benchmark_file%
$load r s h trd f t



* ------------------------------------------------------------------------------
* load parameters
* ------------------------------------------------------------------------------

* benchmark data
parameters
  y0(r,s)               benchmark - output
  ld0(r,s)              benchmark - labor demand
  kd0(r,s)              benchmark - capital demand
  id0(r,s,ss)           benchmark - intermediate demand
  d0(r,s)               benchmark - domestic supply
  m0(r,s,trd)           benchmark - imports
  x0(r,s,trd)           benchmark - exports
  c0(r,h)               benchmark - consumption bundle
  cd0(r,s,h)            benchmark - consumer demand
  i0(r,s)               benchmark - investment demand
  invh0(r,h)            benchmark - household investment
  n0(r,h)               benchmark - households
  g0(r,s)               benchmark - government demand
  le0(r,h)              benchmark - household labor supply
  re0(r,h)              benchmark - capital rental income
  curactbal0            benchmark - current account balance
  inc_row0              benchmark - net income receipts from rest of world
  tran_row0             benchmark - net tax and transfer receipts from rest of world
  deficit0              benchmark - government deficit
  gint0                 benchmark - government interest payments domestically
  gint_row0             benchmark - government interest payments to rest of world
  debt0                 benchmark - government debt
  tran_fed0(r,h)        becnhmark - federal transfer payments to households
  tran_state0(r,h)      becnhmark - state transfer payments to households
  incadj0(r,h)          benchmark - government to household lump sum transfer
  ty0(r,s)              tax rate - production
  tp0(r)                tax rate - payroll tax
  tl0(r,h)              tax rate - labor income tax
  tfica0(r,h)           tax rate - fica tax
  tk0(r)                tax rate - capital income tax
  tc0(r)                tax rate - consumption
  tl_avg0(r,h)          tax rate - average labor income tax;

* Load benchmark data
$load id0 ld0 kd0 cd0 x0 m0 i0 g0 re0 le0 invh0 n0 ty0 tp0 tl0 tfica0 tc0 tk0
$load tl_avg0 tran_fed0 tran_state0 curactbal0 inc_row0 tran_row0 deficit0 gint0
$load gint_row0 debt0

* calculate remaining variables from the original benchmark
y0(r,s)      = (sum(ss, id0(r,ss,s))+(1+tp0(r))*ld0(r,s)+(1+tk0(r))*kd0(r,s))
               /(1-ty0(r,s));
c0(r,h)      = sum(s, cd0(r,s,h))*(1+tc0(r));
incadj0(r,h) = c0(r,h)+invh0(r,h)-re0(r,h)
               -(1-tl_avg0(r,h)-tfica0(r,h))*le0(r,h)
               -(inc_row0+tran_row0-curactbal0)
                  *sum(s, cd0(r,s,h))/sum((rr,s,hh), cd0(rr,s,hh))
               -tran_fed0(r,h)-tran_state0(r,h)
               -(gint0-deficit0)*re0(r,h)/sum((rr,hh), re0(rr,hh));



* ------------------------------------------------------------------------------
* check balance of national sam
* ------------------------------------------------------------------------------

set
  sam_accounts          macro sam accounts / activity, commodity, labor,
                                             capital, household, government,
                                             investment, national, foreign /;

alias (sam_accounts,aa);

parameters
  macro_sam             macro sam;

* build macro sam
macro_sam("activity",   "commodity"  ) = sum((r,s), y0(r,s));
macro_sam("commodity",  "activity"   ) = sum((r,s,ss), id0(r,ss,s));
macro_sam("commodity",  "household"  ) = sum((r,s,h), cd0(r,s,h));
macro_sam("commodity",  "government" ) = sum((r,s), g0(r,s));
macro_sam("commodity",  "investment" ) = sum((r,s), i0(r,s));
macro_sam("commodity",  "national"   ) = sum((r,s), x0(r,s,"dtrd"));
macro_sam("commodity",  "foreign"    ) = sum((r,s), x0(r,s,"ftrd"));
macro_sam("labor",      "activity"   ) = sum((r,s), ld0(r,s));
macro_sam("capital",    "activity"   ) = sum((r,s), kd0(r,s));
macro_sam("household",  "labor"      ) = sum((r,h), le0(r,h));
macro_sam("household",  "capital"    ) = sum((r,h), re0(r,h));
macro_sam("household",  "government" ) = sum((r,h), incadj0(r,h)+tran_fed0(r,h)
                                                   +tran_state0(r,h))+gint0;
macro_sam("household",  "foreign"    ) = inc_row0+tran_row0-curactbal0;
macro_sam("government", "activity"   ) = sum((r,s),  ty0(r,s)*y0(r,s)
                                                    +tk0(r)*kd0(r,s)
                                                    +tp0(r)*ld0(r,s));
macro_sam("government", "household"  ) = sum((r,h), sum(s, tc0(r)*cd0(r,s,h))
                                                    +tl_avg0(r,h)*le0(r,h)
                                                    +tfica0(r,h)*le0(r,h))
                                         +deficit0;
macro_sam("investment", "household"  ) = sum((r,h), invh0(r,h));
macro_sam("national",   "commodity"  ) = sum((r,s), m0(r,s,"dtrd"));
macro_sam("foreign",    "commodity"  ) = sum((r,s), m0(r,s,"ftrd"));
macro_sam("foreign",    "government" ) = gint_row0;

* compute the row and colum totals and check the differences
macro_sam(sam_accounts,"total") = sum(aa, macro_sam(sam_accounts,aa));
macro_sam("total",sam_accounts) = sum(aa, macro_sam(aa,sam_accounts));
macro_sam("r-c diff",sam_accounts) =  macro_sam(sam_accounts,"total")
                                     -macro_sam("total",sam_accounts);

display macro_sam;



* ------------------------------------------------------------------------------
* present basic statistics that can be checked against nipa tables
* ------------------------------------------------------------------------------

parameters
  macro_stats(*,*)      basic statistics for the macro economy;

macro_stats("consumption exp","value") = sum((r,h,s), cd0(r,s,h));
macro_stats("investment exp","value")  = sum((r,s),i0(r,s));
macro_stats("government exp","value")  = sum((r,s), g0(r,s));
macro_stats("exports","value")         = sum((r,s), x0(r,s,"ftrd"));
macro_stats("imports","value")         = sum((r,s), m0(r,s,"ftrd"));
macro_stats("gdp by exp","value")      =  macro_stats("consumption exp","value")
                                         +macro_stats("investment exp","value")
                                         +macro_stats("government exp","value")
                                         +macro_stats("exports","value")
                                         -macro_stats("imports","value");
macro_stats("labor va","value")        = sum((r,s), (1+tp0(r))*ld0(r,s));
macro_stats("capital va","value")      = sum((r,s), (1+tk0(r))*kd0(r,s));
macro_stats("prod tax","value")        = sum((r,s), ty0(r,s)*y0(r,s));
macro_stats("gdp by va","value")       =  macro_stats("labor va","value")
                                         +macro_stats("capital va","value")
                                         +macro_stats("prod tax","value");
macro_stats("fed transfers","value")   = sum((r,h), tran_fed0(r,h));
macro_stats("state transfers","value") = sum((r,h), tran_state0(r,h));
macro_stats("households","value")      = sum((r,h), n0(r,h));

display macro_stats;



* ------------------------------------------------------------------------------
* present basic statistics on the tax system
* ------------------------------------------------------------------------------

parameters
  tax_stats(*,*)        basic statistics on the tax system;

* tax revenue
tax_stats("avg_ty",   "revenue") = sum((r,s), y0(r,s)*ty0(r,s));
tax_stats("avg_tp",   "revenue") = sum((r,s), ld0(r,s)*tp0(r));
tax_stats("avg_tk",   "revenue") = sum((r,s), kd0(r,s)*tk0(r));
tax_stats("avg_tl",   "revenue") = sum((r,h), le0(r,h)*tl_avg0(r,h));
tax_stats("avg_tfica","revenue") = sum((r,h), le0(r,h)*tfica0(r,h));
tax_stats("avg_tc",   "revenue") = sum((r,s,h), cd0(r,s,h)*tc0(r));

* average tax rate
tax_stats("avg_ty",   "rate") = tax_stats("avg_ty",   "revenue")/sum((r,s), y0(r,s));
tax_stats("avg_tp",   "rate") = tax_stats("avg_tp",   "revenue")/sum((r,s), ld0(r,s));
tax_stats("avg_tk",   "rate") = tax_stats("avg_tk",   "revenue")/sum((r,s), kd0(r,s));
tax_stats("avg_tl",   "rate") = tax_stats("avg_tl",   "revenue")/sum((r,h), le0(r,h));
tax_stats("avg_tfica","rate") = tax_stats("avg_tfica","revenue")/sum((r,h), le0(r,h));
tax_stats("avg_tc",   "rate") = tax_stats("avg_tc",   "revenue")/sum((r,s,h), cd0(r,s,h));

display tax_stats;



* ------------------------------------------------------------------------------
* present basic statistics on the distribution of income and expenditures
* ------------------------------------------------------------------------------

parameters
  dist_stats(*,h)       basic statistics on the distribution of income and expenditures;

dist_stats("households",              h) = sum(r, n0(r,h));
dist_stats("before-tax market income",h) = sum(r, (1-tfica0(r,h))*le0(r,h)
                                                  +re0(r,h))/sum(r, n0(r,h));
dist_stats("transfer income",         h) = sum(r, tran_fed0(r,h)
                                                 +tran_state0(r,h))/sum(r, n0(r,h));
dist_stats("incadj",                  h) = sum(r, incadj0(r,h))/sum(r, n0(r,h));
dist_stats("before-tax income",       h) = dist_stats("before-tax market income",h)
                                           +dist_stats("transfer income",h)
                                           +dist_stats("incadj",h);
dist_stats("income tax",              h) = sum(r, tl_avg0(r,h)*le0(r,h))/sum(r, n0(r,h));
dist_stats("after-tax income",        h) = dist_stats("before-tax income",h)
                                           -dist_stats("income tax",h);
dist_stats("consumption",             h) = sum((r,s), cd0(r,s,h))/sum(r, n0(r,h));

display dist_stats;
