## trade_elasticity.R
##
## program calculates GE trade elasticities in the large open economy framework
##
## this example is intended to be run from the top level directory of the model
##



## ------------------------------------------------------------------------------
## load libraries
## ------------------------------------------------------------------------------

## load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
library(ggplot2)
library(dplyr)
library(tidyr)


## ------------------------------------------------------------------------------
## set general options
## ------------------------------------------------------------------------------

## directory to store the images
image.dir = file.path("documentation")

## type of image format in which to save plots
plot.type = "pdf"

## dpi of saved plots
plot.dpi = 150


## ------------------------------------------------------------------------------
## simulate internal export demand and import supply elasticities
## ------------------------------------------------------------------------------

## run the model with an epilogue
run.model(options = paste0("--epilogue=diagnostics/trade_elasticity.gms",
                           " --loe_output_file=output/loe_results.csv"))

## ## read in output file
elasticities = read.csv("output/loe_results.csv",stringsAsFactors=FALSE)

## take average by elasticity, sector, year, and model
avg.elast <- elasticities %>% group_by(elasticity,sector,model,time) %>% summarise(value=mean(value))
avg.elast <- data.frame(avg.elast)
avg.elast.base <- subset(avg.elast, time %in% c("2016"))
avg.elast.base$time <- NULL

## rearrange the data frame
avg.elast.base <- avg.elast.base %>% spread(model,value)
exp <- subset(avg.elast.base, elasticity %in% c("exports"))
exp$elasticity <- NULL
names(exp)[2] <- "gtap.exp"
names(exp)[3] <- "sage.exp"
imp <- subset(avg.elast.base, elasticity %in% c("imports"))
imp$elasticity <- NULL
names(imp)[2] <- "gtap.imp"
names(imp)[3] <- "sage.imp"
avg.elast.base <- merge(exp,imp, by=c("sector"))

## write elasticities to tex file
of = file(file.path(image.dir,paste0("loe_elasticities_base.tex")),open="w")
for (i in 1:nrow(avg.elast.base)) {
    writeLines(paste0(c(avg.elast.base[i,1])," & ",signif(round(avg.elast.base[i,2],2),5)," & ",
                      signif(round(avg.elast.base[i,3],2),5)," & ",signif(round(avg.elast.base[i,5],2),5),
                      " & ",signif(round(avg.elast.base[i,5],2),5),"\\\\")
              ,con=of)
}
close(of)
