* simulates export and import demand elasticities
*
* run as an "epilogue" using:
*   gams model/sage.gms --epilogue=examples/loe_elasticities.gms
*

* ------------------------------------------------------------------------------
* options
* ------------------------------------------------------------------------------

* set the path separator based on the operating system
$iftheni %system.filesys% == UNIX $set sep "/"
$else $set sep "\"
$endif

* set default name for output file if needed
$if not set loe_output_file $set loe_output_file "output%sep%loe_results.csv"



* ------------------------------------------------------------------------------
* generate GE elasticities
* ------------------------------------------------------------------------------

* use savepoint to start model runs from same place
sage.savepoint = 1;

parameters
  e_base(t,s)           baseline exports,
  px_base(t,s)          baseline export price,
  m_base(t,s)           baseline imports,
  pm_base(t,s)          baseline import price;

* save the baseline variables
e_base(t,s)   = xd.l(t,s);
px_base(t,s)  = px.l(t,s);
m_base(t,s)   = ms.l(t,s);
pm_base(t,s)  = pm.l(t,s);

parameters
  ex_dem(t,s),
  im_sup(t,s);

set
  iters                 number of sectors        /is1*is23/;

loop(iters,

* start model runs from baseline
$if exist 'sage_p.gdx' execute_loadpoint 'sage_p.gdx';

* shock the import price
  pm.fx(t,s)$(ord(iters) = ord(s)) = pm_base(t,s)*(1 + 1e-5);

* solve the mcp problem
  solve sage using mcp;

* calculate the GE elasticities
  im_sup(t,s)$((ord(iters) = ord(s)) and m_base(t,s)) =
      (ms.l(t,s)/m_base(t,s)-1)/(pm.l(t,s)/pm_base(t,s)-1+eps);

* reset prices
  pm.lo(t,s)$sum(r, m0(r,s,'ftrd')) = 0;
  pm.up(t,s)$sum(r, m0(r,s,'ftrd')) = inf;
  pm.l(t,s)$sum(r, m0(r,s,'ftrd'))  = pm_base(t,s);

* start model runs from baseline
$if exist 'sage_p.gdx' execute_loadpoint 'sage_p.gdx';

* shock the export price
  px.fx(t,s)$(ord(iters) = ord(s)) = px_base(t,s)*(1 + 1e-5);

* solve the mcp problem
  solve sage using mcp;

* calculate the GE elasticities
  ex_dem(t,s)$((ord(iters) = ord(s)) and e_base(t,s)) =
      (xd.l(t,s)/e_base(t,s)-1)/(px.l(t,s)/px_base(t,s)-1+eps);

* reset prices
  px.lo(t,s)$sum(r, x0(r,s,'ftrd')) = 0;
  px.up(t,s)$sum(r, x0(r,s,'ftrd')) = inf;
  px.l(t,s)$sum(r, x0(r,s,'ftrd'))  = px_base(t,s);

);

parameter
  chk_e                 comparison of ge and gtap elasticities for exports
  chk_i                 comparison of ge and gtap elasticities for imports;

set
  mdl                   models to compare        /sage, gtap/;

chk_e(s,'sage',t)$sum(r, x0(r,s,'ftrd')) = ex_dem(t,s);
chk_e(s,'gtap',t)$(sum(r, x0(r,s,'ftrd')) and chk_e(s,'sage',t)) = eps_ex(s);
chk_i(s,'sage',t)$sum(r, m0(r,s,'ftrd')) = im_sup(t,s);
chk_i(s,'gtap',t)$(sum(r, m0(r,s,'ftrd')) and chk_i(s,'sage',t)) = eps_im(s);

* output csv results file
file loe_results /"%loe_output_file%"/;
loe_results.pc = 5;
loe_results.nd = 6;
put loe_results;

put "elasticity", "sector", "model", "time", "value"/;
loop((s,mdl,t)$chk_e(s,mdl,t), put "exports", s.tl,  mdl.tl, t.tl, chk_e(s,mdl,t) / );
loop((s,mdl,t)$chk_i(s,mdl,t), put "imports", s.tl,  mdl.tl, t.tl, chk_i(s,mdl,t) / );
putclose;
