# national_vs_regional.R
#
# compares the output from the dynamic model with and without regional
# delineation (note that the national model also only includes a single national
# representative agent)
#
# uses the sample regulation in examples/sample_abatement_requirement.gms
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))



# ------------------------------------------------------------------------------
# run the regional model
# ------------------------------------------------------------------------------

# run the baseline
run.model(benchmark_file="default_aggregation.gdx",
          output_file="regional_baseline.csv")

# run the policy scenario
run.model(benchmark_file="default_aggregation.gdx",
          output_file="regional_policy.csv",
          gdx_baseline_file="regional_baseline.gdx",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))



# ------------------------------------------------------------------------------
# run the national model
# ------------------------------------------------------------------------------

# run the baseline
run.model(benchmark_file="national_aggregation.gdx",
          output_file="national_baseline.csv")

# run the policy scenario
run.model(benchmark_file="national_aggregation.gdx",
          output_file="national_policy.csv",
          gdx_baseline_file="national_baseline.gdx",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# load the results
regional.bau    = read.csv(file.path("output","regional_baseline.csv"),
                           stringsAsFactors=FALSE)
regional.policy = read.csv(file.path("output","regional_policy.csv"),
                           stringsAsFactors=FALSE)
national.bau    = read.csv(file.path("output","national_baseline.csv"),
                           stringsAsFactors=FALSE)
national.policy = read.csv(file.path("output","national_policy.csv"),
                           stringsAsFactors=FALSE)

# output equivalent variation based on national file
ev = sum(national.policy$value[national.policy$parameter=="ev_inf"])
printf('EV (national)........................: %7.2f',ev)

# output equivalent variation based on regional file
ev = sum(regional.policy$value[regional.policy$parameter=="ev_inf"])
printf('EV (regional total)..................: %7.2f',ev)

# output the equivalent variation by region
ev = regional.policy[regional.policy$parameter=="ev_inf",]
ev = aggregate(ev$value,by=list(ev$region),sum)
writeLines("EV (regional):")
for (i in 1:nrow(ev))
  writeLines(paste0(ev$Group.1[i],"  ",round(ev$x[i],2)))
