# putty-clay vs putty-putty.R
#
# compares the results of the model when using partial putty-clay capital vs the
# case of fully malleable capital under the putty-putty assumption. plots the
# the output change in the regulated sector under both assumptions.
#
# uses the sample regulation in examples/sample_abatement_requirement.gms
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the expirement under putty-putty
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline_pp.csv",options="--putty_clay=0")

# run the policy scenario
run.model(output_file="policy_pp.csv",options="--putty_clay=0",
          policy_file=file.path("examples","sample_abatement_requirement.gms"),
          gdx_baseline_file="baseline_pp.gdx")



# ------------------------------------------------------------------------------
# run the expirement under partial putty-putty
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline_pc.csv")

# run the policy scenario
run.model(output_file="policy_pc.csv",gdx_baseline_file="baseline_pc.gdx",
          policy_file=file.path("examples","sample_abatement_requirement.gms"))



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# load the results under putty-putty
bau_pp = read.csv(file.path("output","baseline_pp.csv"),stringsAsFactors=FALSE)
policy_pp = read.csv(file.path("output","policy_pp.csv"),stringsAsFactors=FALSE)

# load the results under putty-clay
bau_pc = read.csv(file.path("output","baseline_pc.csv"),stringsAsFactors=FALSE)
policy_pc = read.csv(file.path("output","policy_pc.csv"),stringsAsFactors=FALSE)

# output compensating variation
ev = sum(policy_pp$value[policy_pp$parameter=="ev_inf"])
printf('EV putty-putty............: %7.2f',ev)
ev = sum(policy_pc$value[policy_pc$parameter=="ev_inf"])
printf('EV putty-clay.............: %7.2f',ev)

# set the sector to plot
plot.sector = "pmm"

# variable to plot
plot.variable = "y"

# get the results under the putty-putty assumption
temp_pp = combine.new.extant(bau_pp,plot.variable,plot.sector)
temp_pp$policy = combine.new.extant(policy_pp,plot.variable,plot.sector)$value
temp_pp$scenario = "Putty-Putty"

# get the results under the putty-clay assumption
temp_pc = combine.new.extant(bau_pc,plot.variable,plot.sector)
temp_pc$policy = combine.new.extant(policy_pc,plot.variable,plot.sector)$value
temp_pc$scenario = "Putty-Clay"

# calcualte the percentage change in the national aggregate value
plot_data = rbind(temp_pp,temp_pc)
plot_data = aggregate(plot_data[,c("value","policy")],
                      by=list(plot_data$time,plot_data$scenario),sum)
names(plot_data) = c("time","scenario","value","policy")
plot_data$value = (plot_data$policy/plot_data$value-1)*100

# plot the variable change
p = ggplot(plot_data)+
      geom_line(aes(x=time,y=value,linetype=scenario),size=1.1)+
      labs(x="Year",y=paste0("% Change in ",plot.variable," in ",plot.sector))+
      guides(linetype=guide_legend(title="Scenario"))+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank())

print(p)
