$title sage is an applied general equilibrium model


* ------------------------------------------------------------------------------
* set options
* ------------------------------------------------------------------------------

* set the path separator based on the operating system
$set sep %system.dirsep%

* set default benchmark file
$if not set benchmark_file $set benchmark_file "data%sep%default_aggregation.gdx"

* set default for disaggregating activities based on extant capital
$if not set putty_clay $set putty_clay 1

* set default for parameter file
$if not set parameter_file $set parameter_file "data%sep%parameters.gms"

* set default for baseline file
$if not set baseline_file $set baseline_file "data%sep%baseline.gms"

* by default don't use balanced growth starting values if a baseline is supplied
$if not set balanced_start_values $set balanced_start_values 0

* optional additive perturbation of the y(t,r,s) starting values
$if not set perturb_start $set perturb_start 0

* zero out the baseline file if one is not provided
$if not set gdx_baseline_file $set gdx_baseline_file 0

* zero out the policy file specification if one is not provided
$if not set policy_file $set policy_file 0

* save the results in a gdx file by default
$if not set gdx_save $set gdx_save 1

* default name for the gdx file where the results are saved
$if not set gdx_results_file $set gdx_results_file "output%sep%results.gdx"

* set default name for output file if needed
$if not set output_file $set output_file "output%sep%results.csv"

* zero out the prologue code file specification if one is not provided
$if not set prologue $set prologue 0

* zero out the epilogue code file specification if one is not provided
$if not set epilogue $set epilogue 0

* flag for small vs. large open economy (loe set as default)
$if not set loe $set loe 1

* the following reduce the size of the listing file
$offsymxref
$offsymlist
option limcol = 0;
option limrow = 0;
option solprint = off;



* ------------------------------------------------------------------------------
* define model sets
* ------------------------------------------------------------------------------

* basic model sets
sets
  s                     sectors
  r                     regions
  h                     households
  trd                   trade markets
  f                     primary factors
  t                     time periods
  tfirst(t)             first time period
  tlast(t)              last time period
  v                     capital vintages                  / extant, new /;

* subsets
sets
  ftrd(trd)             foreign trade market
  dtrd(trd)             national trade market
  sm(s)                 materials sectors
  sen(s)                primary energy sectors
  sel(s)                electricity sector
  sstd(s)               standard production sectors
  sres(s)               resource extraction sectors
  sagf(s)               agriculture and forestry sectors
  scm(s)                consumption materials sectors
  scen(s)               consumer primary energy sectors;

* set aliases
alias
  (s,ss), (r,rr), (h,hh), (t,tt);



* ------------------------------------------------------------------------------
* define model parameters
* ------------------------------------------------------------------------------

parameters

* benchmark data
  y0(r,s)               benchmark - output
  rklem0(t,r,s)         benchmark - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  klem0(t,r,s)          benchmark - capital(k)-labor(l)-energy(e)-materials(m) bundle
  kle0(t,r,s)           benchmark - capital(k)-labor(l)-energy(e)-materials(m) bundle
  kl0(t,r,s)            benchmark - capital(k)-labor(l) bundle
  ene0(t,r,s)           benchmark - primary energy(en)-electricity(e) bundle
  en0(t,r,s)            benchmark - primary energy(en) bundle
  mat0(t,r,s)           benchmark - materials(m) bundle
  ld0(r,s)              benchmark - labor demand
  kd0(r,s)              benchmark - capital demand
  id0(r,ss,s)           benchmark - intermediate demand
  ld_base(t,r,s)        benchmark - labor demand
  kd_base(t,r,s)        benchmark - capital demand
  id_base(t,r,ss,s)     benchmark - intermediate demand
  res0(r,s)             benchmark - fixed factor resource
  a0(r,s)               benchmark - armington aggregate
  dn0(r,s)              benchmark - domestic(d)-national(n) bundle
  d0(r,s)               benchmark - domestic supply
  m0(r,s,trd)           benchmark - imports
  x0(r,s,trd)           benchmark - exports
  cl0(r,h)              benchmark - consumption(c)-leisure(l) bundle
  leis0(r,h)            benchmark - leisure
  c0(r,h)               benchmark - consumption bundle
  cd0(r,s,h)            benchmark - consumer demand
  dc0(r,h)              benchmark - discretionary consumption bundle
  dcd0(r,s,h)           benchmark - discretionary demands
  scd0(r,s,h)           benchmark - subsistence demands (non-discretionary)
  cd_base(t,r,s,h)      benchmark - consumer demand
  dcd_base(t,r,s,h)     benchmark - discretionary demand
  scd_base(t,r,s,h)	benchmark - subsistence demands (non-discretionary)
  inv0(r)               benchmark - investment bundle
  i0(r,s)               benchmark - investment demand
  invh0(r,h)            benchmark - household investment
  gov0(r)               benchmark - government bundle
  g0(r,s)               benchmark - government demand
  g_base(t,r,s)         benchmark - government demand
  le0(r,h)              benchmark - labor rental
  l0(r,h)               benchmark - household labor supply
  re0(r,h)              benchmark - capital rental income
  rese0(r,s,h)          benchmark - resource income
  r0(r)                 benchmark - capital rental payments
  curactbal0            benchmark - current account balance
  inc_row0              benchmark - net income receipts from rest of world
  tran_row0             benchmark - net tax and transfer receipts from rest of world
  deficit0              benchmark - government deficit
  gint0                 benchmark - government interest payments domestically
  gint_row0             benchmark - government interest payments to rest of world
  debt0                 benchmark - government debt
  tran_fed0(r,h)        becnhmark - federal transfer payments to households
  tran_state0(r,h)      becnhmark - state transfer payments to households
  g0_share(r,h)         benchmark - share of government exogenous accounts going to household
  oth_adj0(r,h)         benchmark - net of other government adjustments
  res_share(s)          benchmark - fixed factor share of capital
  n0(r,h)               benchmark - households
  pr0                   benchmark - capital rental rate
  fix0(s)               benchmark - fixed factor for exports
  fim0(s)               benchmark - fixed factor for imports
  gdp0                  benchmark - gdp

* benchmark tax rates
  tp0(r)                tax rate - payroll tax
  tl0(r,h)              tax rate - labor income tax
  tfica0(r,h)           tax rate - fica tax
  tk0(r)                tax rate - capital income tax
  tc0(r)                tax rate - consumption
  ty0(r,s)              tax rate - production using new capital
  tl_avg0(r,h)          tax rate - average labor income tax

* tax rates
  tp(t,r)               tax rate - payroll tax
  tl(t,r,h)             tax rate - labor income tax
  tfica(t,r,h)          tax rate - fica tax
  tk(t,r)               tax rate - capital income tax
  tc(t,r,s)             tax rate - consumption
  ty(t,r,s)             tax rate - production using new capital

* elasticities
  se_klem(s)            substitution elasticity - materials and value-added-energy bundle
  se_kl(s)              substitution elasticity - value-added
  se_kle(s)             substitution elasticity - value-added-energy bundle
  se_ene(s)             substitution elasticity - electricity and primary energy bundle
  se_en(s)              substitution elasticity - primary energy bundle
  se_rklem(t,r,s)       substitution elasticity - r-klem bundle in resource extraction sectors
  se_dn(s)              substitution elasticity - domestic-national armington elasticity
  se_nf(s)              substitution elasticity - national-foreign armington elasticity
  se_cl(r,h)            substitution elasticity - consumption-leisure bundle
  se_c                  substitution elasticity - consumption bundle
  se_cen                substitution elasticity - consumption electricity and energy bundle
  se_cene               substitution elasticity - consumption primary energy bundle
  se_cm                 substitution elasticity - consumption materials bundle
  se_inv                substitution elasticity - investment bundle
  eta(r,h)              substitution elasticity - inverse of consumption over time
  te_dx(s)              transformation elasticity - domestic and exported production
  te_k_ex               transformation elasticity - sector differentiated extant capital
  eps_ex                demand elasticity - exports
  eps_im                supply elasticity - imports
  re(s)                 resource elasticity - fixed factor availability

* abatement inputs and demand by region, sector, and vintage of production
  id_abate0(t,r,ss,s,v) abatement - intermediate demand
  ld_abate0(t,r,s,v)    abatement - labor demand
  kd_abate0(t,r,s,v)    abatement - capital demand
  abate0(t,r,s,v)       abatement - abatement demand

* benchmark cost shares
  cs_abate(t,r,*,s,v)   cost share - abatement inputs
  cs_y_ex(t,r,*,s)      cost share - input shares for use with extant capital
  cs_y(t,r,*,s)         cost share - output-abatement bundle
  cs_rklem(t,r,*,s)     cost share - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  cs_klem(t,r,*,s)      cost share - capital(k)-labor(l)-energy(e)-materials(m) bundle
  cs_kle(t,r,*,s)       cost share - capital(k)-labor(l)-energy(e) bundle
  cs_kl(t,r,*,s)        cost share - capital(k)-labor(l) bundle
  cs_ene(t,r,*,s)       cost share - primary energy(en)-electricity(e) bundle
  cs_en(t,r,*,s)        cost share - primary energy(en) bundle
  cs_mat(t,r,sm,s)      cost share - materials(m) bundle
  cs_dn(r,*,s)          cost share - domestic(d)-national(n) armington aggregate
  cs_nf(r,*,s)          cost share - national(n)-foreign(f) armington aggregate
  cs_dx(r,*,s)          cost share - destination differentiated output cet
  cs_w(r,h)             cost share - welfare
  cs_cl(r,h,*)          cost share - consumption(c)-leisure(l) bundle
  cs_c(t,r,h,*)         cost share - consumption bundle
  cs_cene(t,r,h,*)      cost share - consumption electricity and energy bundle
  cs_cen(t,r,h,scen)    cost share - consumption primary energy bundle
  cs_cm(t,r,h,scm)      cost share - consumption materials bundle
  cs_inv(t,r,s)         cost share - investment bundle
  cs_gov(t,r,s)         cost share - government bundle
  cs_kd_ex(r,s)         cost share - sector differentiated extant capital
  cs_loe_x(s)           cost share - open economy exports
  cs_loe_m(s)           cost share - open economy imports
  cs_les(t,r,h,s)       cost share - discretionary spending

* other
  lse_sub               labor supply elasticity - substitution
  lse_inc               labor supply elasticity - income
  se_res_lr(s)          long-run supply elasticity for sectors with fixed factor
  gov_extra(t,r)        additional government expenditures for use in policy file
  incadj_share(r,h)     household share of additional lump-sum transfers
  q_base(t)             quantity index on balanced growth path
  p_base(t)             price index on balanced growth path

* stock data
  extant_share(r,s)     benchmark - share of sector capital that is extant
  extant_share_agg(r)   benchmark - share of total capital that is extant
  k0(r)                 benchmark - capital stock
  ke0(r,h)              benchmark - household capital stock
  k_ex0(r)              benchmark - extant capital

* baseline
  n(t,r,h)              baseline - population
  l_prod_agg(t)         baseline - aggregate labor productivity index
  l_force(t)            baseline - labor force index
  prod_ind(t,r,*,s,v)   baseline - productivity index
  pref_ind(t,r,*,h)     baseline - preference index
  abate_ind(t,r,*,s)    baseline - abatement productivity index
  interval(t)           baseline - years between model periods
  beta(t,r,h)           baseline - discount factor
  te(t,r,h)             baseline - time endowment
  cd_ene_growth(t,s)    baseline - final consumption energy intesity growth rates
  ene_growth(t,ss,s)    baseline - production energy intensity growth rates
  col_ele_growth(t)     baseline - coal share of electricity production growth rate
  curactbal(t)          baseline - current account balance index
  deficit(t)            baseline - deficit index
  debt(t)               baseline - cumulative government debt
  debt_multiplier(t)    baseline - adjusted baseline path with within period deficits
  debt_step_mult(t)     baseline - multiplier for annual linear interpolation of government debt
  gint(t)               baseline - government interest payments domestically
  gint_row(t)           baseline - government interest payments to rest of world
  gov_exp(t)            baseline - government expenditures
  transfers(t)          baseline - government transfer payments
  oth_adj(t)            baseline - net of other government adjustments
  inc_row(t)            baseline - net income receipts from rest of world
  tran_row(t)           baseline - net tax and transfer receipts from rest of world
  int0_share(r,h)       baseline - share of rest of world flows going to regions and households

* rates
  rho(r,h)              rate - pure rate of time preference
  delta                 rate - depreciation rate for new capital
  gamma                 rate - population growth rate
  omega                 rate - labor productivity growth rate
  rbar                  rate - benchmark interest rate
  l_prod(t,s)           rate - sector specific labor productivity growth rate
  l_pref(t)             rate - leisure preference growth rate;



* ------------------------------------------------------------------------------
* define model variables
* ------------------------------------------------------------------------------

positive variables

* prices
  py(t,r,s)             price - output
  pabate(t,r,s,v)       price - abatement
  pres(t,r,s)           price - fixed factor resource
  prklem(t,r,s)         price - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  pklem(t,r,s)          price - capital(k)-labor(l)-energy(e)-materials(m) bundle
  pkle(t,r,s)           price - capital(k)-labor(l)-energy(e) bundle
  pkl(t,r,s)            price - capital(k)-labor(l) bundle
  pmat(t,r,s)           price - materials(m) bundle
  pene(t,r,s)           price - primary energy(en)-electricity(e) bundle
  pen(t,r,s)            price - primary energy(en) bundle
  pa(t,r,s)             price - armington aggregate
  pdn(t,r,s)            price - domestic(d)-national(n) bundle
  pd(t,r,s)             price - domestic supply
  pn(t,s)               price - national market
  pcl(t,r,h)            price - consumption(c)-leisure(l) bundle
  pc(t,r,h)             price - consumption bundle
  pcene(t,r,h)          price - consumption electricity and energy bundle
  pcen(t,r,h)           price - consumption primary energy bundle
  pcm(t,r,h)            price - consumption materials bundle
  pgov(t,r)             price - government bundle
  pfx(t)                price - foreign exchange
  pm(t,s)               price - foreign imports
  px(t,s)               price - foreign exports
  pl(t,r)               price - labor
  pk(t,r)               price - new capital
  pkh(t)                price - share in new capital fund
  pr(t,r)               price - rental rate for new capital
  prh(t)                price - rental rate for household new capital
  pr_ex(t,r,s)          price - rental rate for sector specific extant capital
  pr_ex_agg(t,r)        price - regional rental rate for aggregate extant capital
  prh_ex(t)             price - national rental rate for aggregate extant capital
  pkt(r)                price - terminal capital
  pkht                  price - household terminal capital
  cpi(t)                price - consumer price index
  pfix(t,s)             price - fixed factor export endowment
  pfim(t,s)             price - fixed factor import endowment

* rest of world
  row(t)                income - rest of world

* shadow prices
  lambda(t,r,h)         shadow price - budget constraint

* activities
  y_ex(t,r,s)           activity - output with extant capital
  y(t,r,s)              activity - output with new capital
  a(t,r,s)              activity - armington aggregate
  inv(t,r)              activity - investment bundle
  gov(t,r)              activity - government bundle
  l(t,r,h)              activity - labor

* demands
  abate(t,r,s,v)        demand - abatement
  id_abate(t,r,ss,s,v)  demand - intermediates for abatement
  ld_abate(t,r,s,v)     demand - labor for abatement
  kd_abate(t,r,s,v)     demand - labor for abatement
  ld_ex(t,r,s)          demand - labor for use with extant capital
  kd_ex(t,r,s)          demand - sector specific extant capital
  id_ex(t,r,ss,s)       demand - intermediates for use with extant capital
  res_ex(t,r,s)         demand - fixed factor resource for use with extant capital
  rklem(t,r,s)          demand - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  klem(t,r,s)           demand - capital(k)-labor(l)-energy(e)-materials(m) bundle
  kle(t,r,s)            demand - capital(k)-labor(l)-energy(e) bundle
  kl(t,r,s)             demand - capital(k)-labor(l) bundle
  ene(t,r,s)            demand - primary energy(en)-electricity(e) bundle
  en(t,r,s)             demand - primary energy(en) bundle
  mat(t,r,s)            demand - materials(m) bundle
  ld(t,r,s)             demand - labor
  kd(t,r,s)             demand - capital
  id(t,r,ss,s)          demand - intermediates
  res(t,r,s)            demand - fixed factor resource
  d(t,r,s)              demand - domestic supply
  dn(t,r,s)             demand - domestic(d)-national(n) bundle
  m(t,r,s,trd)          demand - imports
  x(t,r,s,trd)          demand - exports
  cl(t,r,h)             demand - consumption(c)-leisure(l) bundle
  leis(t,r,h)           demand - leisure
  c(t,r,h)              demand - consumption bundle
  dc(t,r,h)             demand - discretionary consumption bundle
  cd(t,r,s,h)           demand - total consumption
  dcd(t,r,s,h)          demand - discretionary consumption
  scd(t,r,s,h)          demand - non-discretionary consumption
  i(t,r,s)              demand - investment
  g(t,r,s)              demand - government
  xd(t,s)               demand - open economy export demand
  fix(t,s)              demand - open economy export fixed factor
  xd_fx(t,s)            demand - open economy export foreign exchange
  ms(t,s)               demand - open economy import supply
  ms_fx(t,s)            demand - open economy import foreign exchange
  fim(t,s)              demand - open economy import fixed factor

* stocks
  k_ex(t,r)             stock - extant capital
  k(t,r)                stock - aggregate new capital
  kh_ex(t,r,h)          stock - household extant capital stock
  kh(t,r,h)             stock - household new capital stock
  kt(r)                 stock - terminal capital
  kht(r,h)              stock - household terminal capital

* unit costs
  uc_abate(t,r,s,v)     unit cost - abatement
  uc_y_ex(t,r,s)        unit cost - output with extant capital
  uc_y(t,r,s)           unit cost - output with new capital
  uc_rklem(t,r,s)       unit cost - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  uc_klem(t,r,s)        unit cost - capital(k)-labor(l)-energy(e)-materials(m) bundle
  uc_kle(t,r,s)         unit cost - capital(k)-labor(l)-energy(e) bundle
  uc_kl(t,r,s)          unit cost - capital(k)-labor(l) bundle
  uc_ene(t,r,s)         unit cost - primary energy(en)-electricity(e) bundle
  uc_en(t,r,s)          unit cost - primary energy(en) bundle
  uc_mat(t,r,s)         unit cost - materials(m) bundle
  uc_a(t,r,s)           unit cost - armington aggregate
  uc_dn(t,r,s)          unit cost - domestic(d)-national(n) armington aggregate
  uc_cl(t,r,h)          unit cost - consumption(c)-leisure(l) bundle
  uc_c(t,r,h)           unit cost - consumption bundle
  uc_cene(t,r,h)        unit cost - consumption electricity and energy bundle
  uc_cen(t,r,h)         unit cost - consumption primary energy bundle
  uc_cm(t,r,h)          unit cost - consumption materials bundle
  uc_inv(t,r)           unit cost - investment bundle
  uc_gov(t,r)           unit cost - government bundle
  uc_loe_x(t,s)         unit cost - open economy exports
  uc_loe_m(t,s)         unit cost - open economy imports

* auxilary
  gdp(t)                auxilary - gdp;

variables

* activity
  invh(t,r,h)           activity - household investment

* auxilary
  incadj(t)             auxilary - government to household lump sum transfer
  inc_extra(t,r,h)      auxilary - additional income relative to numeraire
  w(r,h)                auxilary - welfare;



* ------------------------------------------------------------------------------
* define model equations
* ------------------------------------------------------------------------------

equations

* zero profit conditions
  zp_y_ex(t,r,s)        zero profit - output with extant capital
  zp_y(t,r,s)           zero profit - output with new capital
  zp_abate(t,r,s,v)     zero profit - abatement
  zp_rklem(t,r,s)       zero profit - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  zp_klem(t,r,s)        zero profit - capital(k)-labor(l)-energy(e)-materials(m) bundle
  zp_kle(t,r,s)         zero profit - capital(k)-labor(l)-energy(e) bundle
  zp_kl(t,r,s)          zero profit - capital(k)-labor(l) bundle
  zp_ene(t,r,s)         zero profit - primary energy(en)-electricity(e) bundle
  zp_en(t,r,s)          zero profit - primary energy(en) bundle
  zp_mat(t,r,s)         zero profit - materials(m) bundle
  zp_a(t,r,s)           zero profit - armington aggregate
  zp_dn(t,r,s)          zero profit - domestic(d)-national(n) armington aggregate
  zp_cl(t,r,h)          zero profit - consumption(c)-leisure(l) bundle
  zp_c(t,r,h)           zero profit - consumption bundle
  zp_cene(t,r,h)        zero profit - consumption electricity and energy bundle
  zp_cen(t,r,h)         zero profit - consumption primary energy bundle
  zp_cm(t,r,h)          zero profit - consumption materials bundle
  zp_inv(t,r)           zero profit - investment bundle
  zp_k(t,r)             zero profit - capital stock
  zp_pkh(t)             zero profit - household capital stock
  zp_pkht               zero profit - household terminal capital stock
  zp_gov(t,r)           zero profit - government bundle
  zp_xd_fx(t,s)         zero profit - open economy exports
  zp_ms(t,s)            zero profit - open economy imports

* market clearance conditions
  mc_d(t,r,s)           market clearance - domestic output
  mc_a(t,r,s)           market clearance - goods market
  mc_l(t,r)             market clearance - labor
  mc_time(t,r,h)        market clearance - time
  mc_pr_ex(t,r,s)       market clearance - rental rate for sector differentiated extant capital
  mc_pr(t,r)            market clearance - rental rate for new capital
  mc_pk(t,r)            market clearance - new capital
  mc_invh(t,r,h)        market clearance - household investment
  mc_prh(t)             market clearance - household new capital
  mc_fx(t)              market clearnace - foreign exchange
  mc_dtrd(t,s)          market clearance - national trade
  mc_pkt(r)             market clearance - terminal capital
  mc_pres(t,r,s)        market clearance - fixed factor resources
  mc_px(t,s)            market clearance - open economy exports
  mc_pfix(t,s)          market clearance - open economy exports fixed factor
  mc_pm(t,s)            market clearance - open economy imports
  mc_pfim(t,s)          market clearance - open economy imports fixed factor

* budget constraints
  bc_hh(t,r,h)          budget constraint - household
  bc_gov(t)             budget constraint - government
  bc_row(t)             budget constraint - rest of world

* hicksian demand equations
  hde_ld_ex(t,r,s)      hicksian demand equation - labor for use with extant capital
  hde_kd_ex(t,r,s)      hicksian demand equation - sector specific extant capital
  hde_id_ex(t,r,ss,s)   hicksian demand equation - intermediates for use with extant capital
  hde_res_ex(t,r,s)     hicksian demand equation - fixed factor resources with extant capital
  hde_abate_ex(t,r,s,v) hicksian demand equation - abatement with extant capital
  hde_abate(t,r,s,v)    hicksian demand equation - abatement with new capital
  hde_id_abate(t,r,ss,s,v) hicksian demand equation - intermediates for abatement
  hde_ld_abate(t,r,s,v) hicksian demand equation - labor for abatement
  hde_kd_abate(t,r,s,v) hicksian demand equation - capital for abatement
  hde_rklem(t,r,s)      hicksian demand equation - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  hde_klem(t,r,s)       hicksian demand equation - capital(k)-labor(l)-energy(e)-materials(m) bundle
  hde_kle(t,r,s)        hicksian demand equation - capital(k)-labor(l)-energy(e) bundle
  hde_kl(t,r,s)         hicksian demand equation - capital(k)-labor(l) bundle
  hde_ene(t,r,s)        hicksian demand equation - primary energy(en)-electricity(e) bundle
  hde_en(t,r,s)         hicksian demand equation - primary energy(en) bundle
  hde_mat(t,r,s)        hicksian demand equation - materials(m) bundle
  hde_ld(t,r,s)         hicksian demand equation - labor for use with new capital
  hde_kd(t,r,s)         hicksian demand equation - new capital
  hde_id_ele(t,r,sel,s) hicksian demand equation - electricity intermediates
  hde_id_m(t,r,sm,s)    hicksian demand equation - materials intermediates
  hde_id_en(t,r,sen,s)  hicksian demand equation - primary energy intermediates
  hde_res(t,r,s)        hicksian demand equation - fixed factor resource
  hde_m(t,r,s,trd)      hicksian demand equation - imports
  hde_x(t,r,s,trd)      hicksian demand equation - exports
  hde_d(t,r,s)          hicksian demand equation - domestic supply
  hde_dn(t,r,s)         hicksian demand equation - domestic(d)-national(n) bundle
  hde_leis(t,r,h)       hicksian demand equation - leisure
  hde_c(t,r,h)          hicksian demand equation - aggregate consumption bundle
  hde_dc(t,r,h)         hicksian demand equation - discretionary consumption bundle
  hde_cd(t,r,s,h)       hicksian demand equation - total commodity demand
  hde_dcd(t,r,s,h)      hicksian demand equation - discretionary consumption materials
  hde_scd(t,r,s,h)      hicksian demand equation - non-discretionary consumption materials
  hde_i(t,r,s)          hicksian demand equation - investment
  hde_g(t,r,s)          hicksian demand equation - government
  hde_xd(t,s)           hicksian demand equation - open economy export demand input
  hde_fix(t,s)          hicksian demand equation - open economy export fixed factor
  hde_fim(t,s)          hicksian demand equation - open economy import fixed factor
  hde_ms_fx(t,s)        hicksian demand equation - open economy import foreign exchange

* household first order conditions
  foc_cl(t,r,h)         first order condition - consumption(c)-leisure(l) bundle
  foc_lambda(t,r,h)     first order condition - savings

* unit cost equations
  uce_y_ex(t,r,s)       unit cost equation - output with extant capital
  uce_y(t,r,s)          unit cost equation - output with new capital
  uce_abate(t,r,s,v)    unit cost equation - abatement
  uce_rklem(t,r,s)      unit cost equation - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
  uce_klem(t,r,s)       unit cost equation - capital(k)-labor(l)-energy(e)-materials(m) bundle
  uce_kle(t,r,s)        unit cost equation - capital(k)-labor(l)-energy(e) bundle
  uce_kl(t,r,s)         unit cost equation - capital(k)-labor(l) bundle
  uce_ene(t,r,s)        unit cost equation - primary energy(en)-electricity(e) bundle
  uce_en(t,r,s)         unit cost equation - primary energy(en) bundle
  uce_mat(t,r,s)        unit cost equation - materials(m) bundle
  uce_a(t,r,s)          unit cost equation - armington aggregate
  uce_dn(t,r,s)         unit cost equation - domestic(d)-national(n) armington aggregate
  uce_dx(t,r,s)         unit cost equation - destination differentiated output cet
  uce_k_ex(t,r)         unit cost equation - sector differentiated extant capital
  uce_cl(t,r,h)         unit cost equation - consumption(c)-leisure(l) bundle
  uce_c(t,r,h)          unit cost equation - consumption bundle
  uce_cene(t,r,h)       unit cost equation - consumption electricity and energy bundle
  uce_cen(t,r,h)        unit cost equation - consumption primary energy bundle
  uce_cm(t,r,h)         unit cost equation - consumption materials bundle
  uce_inv(t,r)          unit cost equation - investment bundle
  uce_gov(t,r)          unit cost equation - government bundle
  uce_loe_x(t,s)        unit cost equation - open economy exports
  uce_loe_m(t,s)        unit cost equation - open economy imports

* closure rules
  cr_gov(t,r)           closure rule - real government expenditures fixed
  cr_kt(r)              closure rule - terminal capital
  cr_kht(r,h)           closure rule - household terminal capital

* auxilary
  aux_k_ex(t,r)         auxilary - regional extant capital
  aux_kh_ex(t,r,h)      auxilary - household extant capital
  aux_prh_ex(t)         auxilary - national rental rate on aggregate extant capital
  aux_w(r,h)            auxilary - welfare
  aux_cpi(t)            auxilary - consumer price index
  aux_gdp(t)            auxilary - gdp;



* ------------------------------------------------------------------------------
* load sets and define subsets
* ------------------------------------------------------------------------------

$gdxin %benchmark_file%
$load r s h trd f

* define subsets
sets
  ftrd(trd)             foreign trade market             / ftrd /
  dtrd(trd)             national trade market            / dtrd /
  sm(s)                 materials sectors                / agf, cru, min, wsu,
                                                           con, fbm, wpm, chm,
                                                           prm, cem, pmm,
                                                           fmm, cpu, tem, bom,
                                                           trn, ttn, srv, hlt /
  sen(s)                primary energy sectors           / col, gas, ref /
  sel(s)                electricity sector               / ele /
  sstd(s)               standard production sectors      / ele, wsu, con, fbm,
                                                           wpm, ref, chm, prm,
                                                           cem, pmm, fmm,
                                                           cpu, tem, bom, trn,
                                                           ttn, srv, hlt /
  sres(s)               resource extraction sectors      / cru, gas, col, min /
  sagf(s)               agriculture and forestry sectors / agf /
  scm(s)                consumption materials sectors    / agf, cru, min, wsu,
                                                           con, fbm, wpm, chm,
                                                           prm, cem, pmm,
                                                           fmm, cpu, tem, bom,
                                                           trn, ttn, srv, hlt /
  scen(s)               consumption energy sectors       / col, gas, ref /;



* ------------------------------------------------------------------------------
* load benchmark data
* ------------------------------------------------------------------------------

* load benchmark IMPLAN data
$load id0 ld0 kd0 cd0 scd0 x0 m0 i0 g0 re0 le0 invh0 n0 ty0 tp0 tl0 tfica0 tc0
$load tk0 tl_avg0 tran_fed0 tran_state0 curactbal0 inc_row0 tran_row0 deficit0
$load gint0 gint_row0 debt0



* ------------------------------------------------------------------------------
* load the parameters and baseline
* ------------------------------------------------------------------------------

* default optional parameters to zero (i.e., default is to not add features)
res_share(s)        = 0;
se_res_lr(s)        = 0;
re(s)               = 0;

* load in parameters
$include %parameter_file%

* load in baseline
$include %baseline_file%



* ------------------------------------------------------------------------------
* define special temporal sets
* ------------------------------------------------------------------------------

* special sets defining key time periods
tfirst(t) = yes$(ord(t) eq 1);
tlast(t)  = yes$(ord(t) eq card(t));



* ------------------------------------------------------------------------------
* update rates for the model interval and define reference paths
* ------------------------------------------------------------------------------

* get the interval between model periods
if (card(t) eq 1,
  interval(t) = 1;
else
  interval(t) = sum(tt$(ord(tt) eq ord(t)+1), tt.val)-t.val;
  interval(t)$tlast(t) = interval(t-1);
);

* discount factor via ramsey formula - this formulation accounts for the fact
* that the more traditional relationship rbar = rho+eta*(omega+gamma) would only
* hold for small time steps
rho(r,h)     = sum(t$(ord(t) eq 4),
                (1+rbar)/((l_prod_agg(t)**(1/(t.val-sum(tfirst, tfirst.val)))
                          -1+l_force(t)**(1/(t.val-sum(tfirst, tfirst.val))))
                          /(l_force(t)**(1/(t.val-sum(tfirst, tfirst.val))))
                         )**eta(r,h)-1);
beta(t,r,h)        = (1/(1+rho(r,h)))**(t.val-sum(tfirst, tfirst.val));

* reference quantities on balanced growth path - grow at steady state rate
q_base(t) = l_prod_agg(t)*l_force(t);

* reference prices on balanced growth path - decline with the interest rate
p_base(t) = 1/(1+rbar)**(t.val-sum(tfirst, tfirst.val));



* ------------------------------------------------------------------------------
* run prologue code if provided
* ------------------------------------------------------------------------------

$if %prologue%==0 $goto prologue_end

$include %prologue%

$label prologue_end



* ------------------------------------------------------------------------------
* disaggregate land and natural resource payments
* ------------------------------------------------------------------------------

* resource payments for new capital production
res0(r,s)    = res_share(s)*kd0(r,s);
kd0(r,s)     = kd0(r,s)-res0(r,s);
rese0(r,s,h) = res0(r,s)*re0(r,h)/sum(hh, re0(r,hh));

* recalculate household capital rental income
re0(r,h)    = re0(r,h)-sum(s, rese0(r,s,h));



* ------------------------------------------------------------------------------
* set extant share of benchmark capital
* ------------------------------------------------------------------------------

* share of first period capital stock that is extant
if (%putty_clay% eq 1 and card(t) gt 1,

* all capital in the first period is extant capital
  extant_share(r,s) = 1;

else

* if model is static (one period) all capital is malleable
  extant_share(r,s) = 0;

);

* benchmark - share of total capital that is extant
extant_share_agg(r) = sum(s, extant_share(r,s)*kd0(r,s))/sum(s, kd0(r,s));



* ------------------------------------------------------------------------------
* baseline cost share adjustments based on energy efficiency improvements
* ------------------------------------------------------------------------------

* initialize discretionary component of final demand
dcd0(r,s,h) = cd0(r,s,h)-scd0(r,s,h);

* initialize the benchmark inputs and final demand
ld_base(t,r,s)    = ld0(r,s);
kd_base(t,r,s)    = kd0(r,s);
id_base(t,r,ss,s) = id0(r,ss,s);
cd_base(t,r,s,h)  = cd0(r,s,h);
dcd_base(t,r,s,h) = dcd0(r,s,h);
scd_base(t,r,s,h) = scd0(r,s,h);
g_base(t,r,s)     = g0(r,s);

* apply changes in the consumption shares where all changes are balanced by
* sharing out the change across the remaining sectors that were not
* changed. assume discretionary demands follow the same changes as aggregate
* commodity demand
dcd_base(t,r,s,h)$(not cd_ene_growth(t,s) and dcd0(r,s,h))
  = dcd_base(t,r,s,h)
    +sum(ss$cd_ene_growth(t,ss), (1-cd_ene_growth(t,ss))
                                 *dcd_base(t,r,ss,h))*dcd_base(t,r,s,h)
     /sum(ss$(not cd_ene_growth(t,ss)), dcd_base(t,r,ss,h));
dcd_base(t,r,s,h)$(cd_ene_growth(t,s) and dcd0(r,s,h)) = cd_ene_growth(t,s)
                                                         *dcd_base(t,r,s,h);

* assume that subsistence demands follow the same changes as aggregate commodity demand
scd_base(t,r,s,h)$(not cd_ene_growth(t,s) and scd0(r,s,h))
  = scd_base(t,r,s,h)
    +sum(ss$cd_ene_growth(t,ss), (1-cd_ene_growth(t,ss))
                                 *scd_base(t,r,ss,h))*scd_base(t,r,s,h)
     /sum(ss$(not cd_ene_growth(t,ss)), scd_base(t,r,ss,h));
scd_base(t,r,s,h)$(cd_ene_growth(t,s) and scd0(r,s,h)) = cd_ene_growth(t,s)
                                                         *scd_base(t,r,s,h);

* require discretionary and subsistence demands to add up
cd_base(t,r,s,h) = dcd_base(t,r,s,h)+scd_base(t,r,s,h);

* for government consumption apply the same energy intensity assumptions as in
* household consumption
g_base(t,r,s)$(not cd_ene_growth(t,s))
  = g_base(t,r,s)
    +sum(ss$cd_ene_growth(t,ss), (1-cd_ene_growth(t,ss))
                                 *g_base(t,r,ss))*g_base(t,r,s)
     /sum(ss$(not cd_ene_growth(t,ss)), g_base(t,r,ss));
g_base(t,r,s)$cd_ene_growth(t,s) = cd_ene_growth(t,s)*g_base(t,r,s);

* apply energy intesity improvemnts as a change in the cost shares associated
* with energy inputs assuming that the changes are capital embodied resulting
* in a larger capital share
parameter
  ene_factor(t,r,ss,s)  energy intensity growth factor;
ene_factor(t,r,ss,s) = 1;
ene_factor(t,r,ss,s)$(not tfirst(t) and ene_growth(t,ss,s))
  = (ene_growth(t,ss,s)*q_base(t)
     -extant_share(r,s)*(1-delta)**(t.val-sum(tfirst, tfirst.val)))
    /(q_base(t)-extant_share(r,s)*(1-delta)**(t.val-sum(tfirst, tfirst.val)));

kd_base(t,r,s) = kd_base(t,r,s)
                 +sum(ss, (1-ene_factor(t,r,ss,s))*id_base(t,r,ss,s))
                  /(1+tk0(r));
id_base(t,r,ss,s) = ene_factor(t,r,ss,s)*id_base(t,r,ss,s);

* adjust the coal fired electricity generation share assuming that capital and
* labor take up the slack proportionally as a proxy for renewables
parameter
  col_ele_factor(t,r)   coal input share growth factor ;
col_ele_factor(t,r) = 1;
col_ele_factor(t,r)$(not tfirst(t))
  = (col_ele_growth(t)*q_base(t)
     -sum(sel, extant_share(r,sel))*(1-delta)**(t.val-sum(tfirst, tfirst.val)))
    /(q_base(t)-sum(sel, extant_share(r,sel))*(1-delta)**(t.val-sum(tfirst, tfirst.val)));

parameter
  k_share_col(t,r)      capital embodied share of reduced coal expenditures;
k_share_col(t,r) = sum(s$sel(s), (1+tk0(r))*kd_base(t,r,s)
                                 /( (1+tk0(r))*kd_base(t,r,s)
                                   +(1+tp0(r))*ld_base(t,r,s)));

kd_base(t,r,s)$(sel(s)) = kd_base(t,r,s)
                          +(1-col_ele_factor(t,r))
                          *id_base(t,r,"col",s)*k_share_col(t,r)/(1+tk0(r));
ld_base(t,r,s)$(sel(s)) = ld_base(t,r,s)
                          +(1-col_ele_factor(t,r))
                          *id_base(t,r,"col",s)*(1-k_share_col(t,r))/(1+tp0(r));
id_base(t,r,"col",sel) = col_ele_factor(t,r)*id_base(t,r,"col",sel);



* ------------------------------------------------------------------------------
* calculate remaining benchmark variables
* ------------------------------------------------------------------------------

* commodity bundles in production functions
kl0(t,r,s)         = (1+tp0(r))*ld_base(t,r,s)+(1+tk0(r))*kd_base(t,r,s);
en0(t,r,s)         = sum(sen, id_base(t,r,sen,s));
ene0(t,r,s)        = sum(sel, id_base(t,r,sel,s))+en0(t,r,s);
kle0(t,r,s)        = kl0(t,r,s)+ene0(t,r,s);
mat0(t,r,s)        = sum(sm, id_base(t,r,sm,s));
klem0(t,r,s)       = sum(ss, id_base(t,r,ss,s))+kl0(t,r,s);
rklem0(t,r,s)      = klem0(t,r,s)+(1+tk0(r))*res0(r,s);

* output
y0(r,s)            = ((1+tp0(r))*ld0(r,s)+(1+tk0(r))*kd0(r,s)
                      +sum(ss, id0(r,ss,s))+(1+tk0(r))*res0(r,s))
                     /(1-ty0(r,s));

* domestic own-use and armington aggregates
d0(r,s)            = y0(r,s)-sum(trd, x0(r,s,trd));
dn0(r,s)           = d0(r,s)+m0(r,s,"dtrd");
a0(r,s)            = dn0(r,s)+m0(r,s,"ftrd");

* discretionary commodity bundles for utility functions (discretionary spending)
dc0(r,h)           = sum(s, dcd0(r,s,h))*(1+tc0(r));

* aggregate commodity demand (discretionary + subsistence)
c0(r,h)            = sum(s, cd0(r,s,h))*(1+tc0(r));

* aggregate regional capitral rents and investment
r0(r)              = sum(s, kd0(r,s));
inv0(r)            = sum(s, i0(r,s));

* aggregate regional government expenditures
gov0(r)            = sum(s, g0(r,s));

* benchmark rental price for capital (defined using annual rates)
pr0                = rbar+delta;

* benchmark capital stock
k0(r)              = r0(r)/pr0;

* benchmark capital endowment
ke0(r,h)           = re0(r,h)/pr0;

* convert capital rental payments to capital stock
kd0(r,s)           = kd0(r,s)/pr0;
kd_base(t,r,s)     = kd_base(t,r,s)/pr0;

* household labor income
l0(r,h)            = le0(r,h);

* benchmark - share of exogenous government accounts going to household
g0_share(r,h)      = c0(r,h)/sum((rr,hh), c0(rr,hh));

* baseline - share of rest of world flows going to regions and households
int0_share(r,h)    = sum((ftrd,s), m0(r,s,ftrd)-x0(r,s,ftrd))
                       /sum((ftrd,rr,s), m0(rr,s,ftrd)-x0(rr,s,ftrd))
                     *c0(r,h)/sum(hh, c0(r,hh));

* implicit remaining government net transfers
oth_adj0(r,h)       = c0(r,h)+invh0(r,h)
                      -(1-tl_avg0(r,h)-tfica0(r,h))*le0(r,h)
                      -(re0(r,h)+sum(s, rese0(r,s,h)))
                      -inc_row0*int0_share(r,h)
                      -tran_row0*int0_share(r,h)
                      +curactbal0*int0_share(r,h)
                      -tran_fed0(r,h)-tran_state0(r,h)
                      -(gint0-deficit0)*g0_share(r,h);

* benchmark - gdp
gdp0 = sum((r,s,h), cd0(r,s,h))+sum((r,s), i0(r,s))+sum((r,s), g0(r,s))
       +sum((r,s), x0(r,s,"ftrd"))-sum((r,s), m0(r,s,"ftrd"));



* ------------------------------------------------------------------------------
* set consumption-leisure substitution elasticity and benchmark leisure
* ------------------------------------------------------------------------------

* compute benchmark leisure and the c-l substitution elasticity
leis0(r,h)         = -dc0(r,h)*lse_inc/((1+lse_inc)*(1-tl0(r,h)-tfica0(r,h))
                                        +tl0(r,h)-tl_avg0(r,h));
le0(r,h)           = l0(r,h)+leis0(r,h);
cl0(r,h)           = dc0(r,h)+(1-tl0(r,h)-tfica0(r,h))*leis0(r,h);
se_cl(r,h)         = lse_sub*cl0(r,h)/dc0(r,h)*l0(r,h)/leis0(r,h);



* ------------------------------------------------------------------------------
* define additional parameters
* ------------------------------------------------------------------------------

* use benchmark tax rates as defaults
tp(t,r)            = tp0(r);
tl(t,r,h)          = tl0(r,h);
tfica(t,r,h)       = tfica0(r,h);
tk(t,r)            = tk0(r);
tc(t,r,s)          = tc0(r);
ty(t,r,s)          = ty0(r,s);

* household share of additional lump-sum transfers
* lump-sum transfers (incadj) are a national variable used to balance the
* government budget constraint - this parameter determines how that transfer is
* shared out across regions and households
incadj_share(r,h) = g0_share(r,h);

* set additional real government spending to zero by default - option present
* for policy scenarios only
gov_extra(t,r) = 0;



* ------------------------------------------------------------------------------
* calculate cost shares
* ------------------------------------------------------------------------------

* note that any cost shares dependent on potential abatement requirements are
* defined after the policy file is loaded - this includes cs_abate, cs_y, and
* cs_y_ex

* compute cost shares
cs_kd_ex(r,s)           = (extant_share(r,s)*kd0(r,s)          / (extant_share_agg(r)*k0(r))  )$(extant_share(r,s) and y0(r,s));
cs_rklem(t,r,"res",s)   = ((1+tk0(r))*res0(r,s)                / rklem0(t,r,s)                )$(rklem0(t,r,s));
cs_rklem(t,r,"klem",s)  = (klem0(t,r,s)                        / rklem0(t,r,s)                )$(rklem0(t,r,s));
cs_klem(t,r,"mat",s)    = (mat0(t,r,s)                         / klem0(t,r,s)                 )$(klem0(t,r,s));
cs_klem(t,r,"kle",s)    = (kle0(t,r,s)                         / klem0(t,r,s)                 )$(klem0(t,r,s));
cs_kle(t,r,"ene",s)     = (ene0(t,r,s)                         / kle0(t,r,s)                  )$(kle0(t,r,s));
cs_kle(t,r,"kl",s)      = (kl0(t,r,s)                          / kle0(t,r,s)                  )$(kle0(t,r,s));
cs_kl(t,r,"k",s)        = ((1+tk0(r))*pr0*kd_base(t,r,s)       / kl0(t,r,s)                   )$(kl0(t,r,s));
cs_kl(t,r,"l",s)        = ((1+tp0(r))*ld_base(t,r,s)           / kl0(t,r,s)                   )$(kl0(t,r,s));
cs_ene(t,r,sel,s)       = (id_base(t,r,sel,s)                  / ene0(t,r,s)                  )$(ene0(t,r,s));
cs_ene(t,r,"en",s)      = (en0(t,r,s)                          / ene0(t,r,s)                  )$(ene0(t,r,s));
cs_en(t,r,sen,s)        = (id_base(t,r,sen,s)                  / en0(t,r,s)                   )$(en0(t,r,s));
cs_mat(t,r,sm,s)        = (id_base(t,r,sm,s)                   / mat0(t,r,s)                  )$(mat0(t,r,s));
cs_dn(r,"d",s)          = (d0(r,s)                             / dn0(r,s)                     )$(dn0(r,s));
cs_dn(r,"n",s)          = (m0(r,s,"dtrd")                      / dn0(r,s)                     )$(dn0(r,s));
cs_nf(r,"n",s)          = (dn0(r,s)                            / a0(r,s)                      )$(a0(r,s));
cs_nf(r,"f",s)          = (m0(r,s,"ftrd")                      / a0(r,s)                      )$(a0(r,s));
cs_dx(r,"d",s)          = (d0(r,s)                             / y0(r,s)                      )$(y0(r,s));
cs_dx(r,"n",s)          = (x0(r,s,"dtrd")                      / y0(r,s)                      )$(y0(r,s));
cs_dx(r,"f",s)          = (x0(r,s,"ftrd")                      / y0(r,s)                      )$(y0(r,s));
cs_cl(r,h,"leis")       = ((1-tl0(r,h)-tfica0(r,h))*leis0(r,h) / cl0(r,h)                     )$(cl0(r,h));
cs_cl(r,h,"c")          = (dc0(r,h)                            / cl0(r,h)                     )$(cl0(r,h));
cs_inv(t,r,s)           = (i0(r,s)                             / inv0(r)                      )$(inv0(r));
cs_gov(t,r,s)           = (g_base(t,r,s)                       / gov0(r)                      )$(gov0(r));
cs_les(t,r,h,s)         = (dcd_base(t,r,s,h)                   / dc0(r,h)                     )$(dc0(r,h));



* ------------------------------------------------------------------------------
* calibrate large open economy parameters
* ------------------------------------------------------------------------------

* set the cost shares for imports and exports "fixed factor" so the slope of the
* supply and demand curves, respectively, match the specified elasticities at
* the benchmark equilibrium - if not the large open economy flag is not set, set
* the shares to zero to collapse to a small open economy assumption
$if %loe%==1 cs_loe_x(s) = 1/(abs(eps_ex(s)));
$if %loe%==0 cs_loe_x(s) = 0;
$if %loe%==1 cs_loe_m(s) = 1/(1+eps_im(s));
$if %loe%==0 cs_loe_m(s) = 0;

* fixed import and export factors for row agent
fix0(s) = cs_loe_x(s)/(1-cs_loe_x(s))*sum((r,ftrd), x0(r,s,ftrd));
fim0(s) = cs_loe_m(s)*sum((r,ftrd), m0(r,s,ftrd));



* ------------------------------------------------------------------------------
* calibrate resource substitution elasticity in sectors with fixed factors
* ------------------------------------------------------------------------------

* substitution elasticity - r-klem bundle in resource extraction sectors
* calibrate the price elasticity of supply to be equal to se_res_lr given the
* benchmark input shares
se_rklem(t,r,s)$cs_rklem(t,r,"res",s) = se_res_lr(s)
                                        /(1/cs_rklem(t,r,"res",s)-1);



* ------------------------------------------------------------------------------
* define the baseline
* ------------------------------------------------------------------------------

* baseline household population grows at rate gamma
n(t,r,h)  = l_force(t)*n0(r,h);

* time endowment based on population growth
te(t,r,h) = l_force(t)*le0(r,h);

* initialize productivity index to unity
prod_ind(t,r,ss,s,v)      = 1;
prod_ind(t,r,"k",s,v)     = 1;
prod_ind(t,r,"l",s,v)     = 1;

* initialize productivity index for abatement activities to unity
abate_ind(t,r,ss,s)       = 1;
abate_ind(t,r,"k",s)      = 1;
abate_ind(t,r,"l",s)      = 1;

* intialize preference index to unity
pref_ind(t,r,s,h)         = 1;
pref_ind(t,r,"l",h)       = 1;

* initalize abatement demands to zero
id_abate0(t,r,ss,s,v) = 0;
ld_abate0(t,r,s,v)    = 0;
kd_abate0(t,r,s,v)    = 0;

* sector specific labor productivity growth
prod_ind(t,r,"l",s,"extant") = 1/l_prod(t,s);
prod_ind(t,r,"l",s,"new") = 1/l_prod(t,s);

* abatement activity labor productivity growth
abate_ind(t,r,"l",s) = 1/l_prod_agg(t);

* leisure productivity growth
pref_ind(t,r,"l",h) = 1/l_prod_agg(t);

* current period deficit is included in current period debt per cbo. debt is
* calculated as the cumulative total of the previous time period debt plus the
* sum of deficits in between the current period and previous period. when the
* model is specified using multi-year time steps, we linearly interpolate
* between the model years to calculate cumulative debt.
set debt_counter / 1*100 /;
debt_step_mult(t) = sum(debt_counter$(debt_counter.val le interval(t)),
    debt_counter.val);

* account for within time step deficits linearly
debt_multiplier(t) = 1$(ord(t) eq 1) + 
    ((interval(t)*deficit0*deficit(t-1)
	+debt_step_mult(t)*(deficit0*deficit(t)-deficit0*deficit(t-1))
	/interval(t))$(ord(t) ne card(t) and ord(t) ne 1)
	+(deficit0*deficit(t))$(ord(t) eq card(t))) /
    deficit0;

* construct baseline path debt muliplier
debt(t) = (debt0 + sum((tt)$(ord(tt) le ord(t) and ord(tt) ne 1),
    deficit0*debt_multiplier(tt))) / debt0;



* ------------------------------------------------------------------------------
* unit cost functions
* ------------------------------------------------------------------------------

* unit cost equation - output with extant capital
uce_y_ex(t,r,s)$(extant_share(r,s) and y0(r,s))..
  uc_y_ex(t,r,s) =e= (sum(ss, cs_y_ex(t,r,ss,s)*prod_ind(t,r,ss,s,"extant")
                              *pa(t,r,ss))
                      +cs_y_ex(t,r,"l",s)*prod_ind(t,r,"l",s,"extant")
                        *(1+tp(t,r))*pl(t,r)/(1+tp0(r))
                      +cs_y_ex(t,r,"k",s)*prod_ind(t,r,"k",s,"extant")
                        *(1+tk(t,r))*pr_ex(t,r,s)/((1+tk0(r))*pr0)
                      +cs_y_ex(t,r,"res",s)*(1+tk(t,r))*pres(t,r,s)/(1+tk0(r))
                      +cs_y_ex(t,r,"abate",s)*pabate(t,r,s,"extant")
                     )/(1-cs_y_ex(t,r,"abate",s))*(1-ty0(r,s));

* unit cost equation - output with abatement using new capital
uce_y(t,r,s)$(y0(r,s))..
  uc_y(t,r,s) =e= (cs_y(t,r,"abate",s)/cs_y(t,r,"rklem",s)*pabate(t,r,s,"new")
                   +prklem(t,r,s))*(1-ty0(r,s));

* unit cost equation - abatement
uce_abate(t,r,s,v)$(abate0(t,r,s,v))..
  uc_abate(t,r,s,v) =e= sum(ss, cs_abate(t,r,ss,s,v)*pa(t,r,ss)
                                *abate_ind(t,r,ss,s))
                        +cs_abate(t,r,"l",s,v)*abate_ind(t,r,"l",s)
                           *(1+tp(t,r))*pl(t,r)/(1+tp0(r))
                        +cs_abate(t,r,"k",s,v)*abate_ind(t,r,"k",s)
                           *(1+tk(t,r))*pr(t,r)/((1+tk0(r))*pr0);

* unit cost equation - resource(r)-capital(k)-labor(l)-energy(e)-materials(m)
* bundle
uce_rklem(t,r,s)$(y0(r,s))..
  uc_rklem(t,r,s) =e= ( cs_rklem(t,r,"res", s)*((1+tk(t,r))*pres(t,r,s)
                                                /(1+tk0(r))
                                               )**(1-se_rklem(t,r,s))
                       +cs_rklem(t,r,"klem",s)*pklem(t,r,s)**(1-se_rklem(t,r,s))
                      )**(1/(1-se_rklem(t,r,s)));

* unit cost equation - capital(k)-labor(l)-energy(e)-materials(m) bundle
uce_klem(t,r,s)$(y0(r,s))..
  uc_klem(t,r,s) =e= ( cs_klem(t,r,"mat",s)*pmat(t,r,s)**(1-se_klem(s))
                      +cs_klem(t,r,"kle",s)*pkle(t,r,s)**(1-se_klem(s))
                     )**(1/(1-se_klem(s)));

* unit cost equation - capital(k)-labor(l)-energy(e) bundle
uce_kle(t,r,s)$(y0(r,s))..
  uc_kle(t,r,s) =e= ( cs_kle(t,r,"ene",s)*pene(t,r,s)**(1-se_kle(s))
                     +cs_kle(t,r,"kl",s) *pkl(t,r,s) **(1-se_kle(s))
                    )**(1/(1-se_kle(s)));

* unit cost equation - capital(k)-labor(l) bundle
uce_kl(t,r,s)$(y0(r,s) and not (tfirst(t) and extant_share(r,s) eq 1))..
  uc_kl(t,r,s) =e= ( cs_kl(t,r,"l",s)*(prod_ind(t,r,"l",s,"new")
                                       *(1+tp(t,r))*pl(t,r)/(1+tp0(r))
                                      )**(1-se_kl(s))
                    +cs_kl(t,r,"k",s)*(prod_ind(t,r,"k",s,"new")
                                       *(1+tk(t,r))*pr(t,r)/((1+tk0(r))*pr0)
                                      )**(1-se_kl(s))
                   )**(1/(1-se_kl(s)));

* unit cost equation - primary energy(en)-electricity(e) bundle
uce_ene(t,r,s)$(y0(r,s))..
  uc_ene(t,r,s) =e= ( cs_ene(t,r,"ele",s)*(sum(ss$sel(ss),
                                                 prod_ind(t,r,ss,s,"new")
                                                 *pa(t,r,ss))
                                          )**(1-se_ene(s))
                     +cs_ene(t,r,"en",s)*pen(t,r,s)**(1-se_ene(s))
                    )**(1/(1-se_ene(s)));

* unit cost equation - primary energy(en) bundle
uce_en(t,r,s)$(y0(r,s))..
  uc_en(t,r,s) =e= sum(sen, cs_en(t,r,sen,s)*(prod_ind(t,r,sen,s,"new")
                                              *pa(t,r,sen))**(1-se_en(s)))
                   **(1/(1-se_en(s)));

* unit cost equation - materials(m) bundle
uce_mat(t,r,s)$(y0(r,s))..
  uc_mat(t,r,s) =e= sum(sm, cs_mat(t,r,sm,s)*prod_ind(t,r,sm,s,"new")
                            *pa(t,r,sm));

* unit cost equation - armington aggregate
uce_a(t,r,s)..
  uc_a(t,r,s) =e= ( cs_nf(r,"n",s)*pdn(t,r,s)**(1-se_nf(s))
                   +cs_nf(r,"f",s)*(pm(t,s)) **(1-se_nf(s))
                  )**(1/(1-se_nf(s)));

* unit cost equation - domestic-national armington aggregate
uce_dn(t,r,s)..
  uc_dn(t,r,s) =e= ( (cs_dn(r,"d",s)*pd(t,r,s)**(1-se_dn(s)))$(cs_dn(r,"d",s))
                    +(cs_dn(r,"n",s)*pn(t,s)  **(1-se_dn(s)))$(cs_dn(r,"n",s))
                   )**(1/(1-se_dn(s)));

* unit cost equation - destination differentiated output cet
uce_dx(t,r,s)$(y0(r,s))..
  py(t,r,s) =e= ( cs_dx(r,"d",s)*pd(t,r,s)**(1+te_dx(s))
                 +cs_dx(r,"n",s)*pn(t,s)  **(1+te_dx(s))
                 +cs_dx(r,"f",s)*px(t,s)  **(1+te_dx(s))
                )**(1/(1+te_dx(s)));

* unit cost equation - sector differentiated extant capital
uce_k_ex(t,r)$(extant_share_agg(r))..
  pr_ex_agg(t,r)/pr0 =e= sum(s, cs_kd_ex(r,s)*(pr_ex(t,r,s)/pr0)**(1+te_k_ex))
                     **(1/(1+te_k_ex));

* unit cost equation - consumption(c)-leisure(l) bundle
uce_cl(t,r,h)..
  uc_cl(t,r,h) =e= ( cs_cl(r,h,"leis")*((1-tl(t,r,h)-tfica(t,r,h))
                                        *pref_ind(t,r,"l",h)*pl(t,r)
                                        /(1-tl0(r,h)-tfica0(r,h))
                                       )**(1-se_cl(r,h))
                    +cs_cl(r,h,"c")*pc(t,r,h)**(1-se_cl(r,h))
                   )**(1/(1-se_cl(r,h)));

* unit cost equation - consumption bundle
uce_c(t,r,h)..
  uc_c(t,r,h) =e= prod(s, (pref_ind(t,r,s,h)*(1+tc(t,r,s))*pa(t,r,s)/(1+tc0(r)))
                    **cs_les(t,r,h,s));

* unit cost equation - investment bundle
uce_inv(t,r)..
  uc_inv(t,r) =e= sum(s, cs_inv(t,r,s)*pa(t,r,s)**(1-se_inv))**(1/(1-se_inv));

* unit cost equation - government bundle
uce_gov(t,r)..
  uc_gov(t,r) =e= sum(s, cs_gov(t,r,s)*pa(t,r,s));

* unit cost equation - open economy exports
uce_loe_x(t,s)..
  uc_loe_x(t,s) =e= pfix(t,s)**cs_loe_x(s)*px(t,s)**(1-cs_loe_x(s));

* unit cost equation - open economy imports
uce_loe_m(t,s)..
  uc_loe_m(t,s) =e= pfim(t,s)**cs_loe_m(s)*pfx(t)**(1-cs_loe_m(s));



* ------------------------------------------------------------------------------
* hicksian demand functions - via shepard's lemma
* ------------------------------------------------------------------------------

* hicksian demand equation - labor for use with extant capital
hde_ld_ex(t,r,s)$(extant_share(r,s) and ld0(r,s))..
  ld_ex(t,r,s) =e= prod_ind(t,r,"l",s,"extant")*y_ex(t,r,s);

* hicksian demand equation - sector specific extant capital
hde_kd_ex(t,r,s)$(extant_share(r,s) and kd0(r,s))..
  kd_ex(t,r,s) =e= prod_ind(t,r,"k",s,"extant")*y_ex(t,r,s);

* hicksian demand equation - intermediates for use with extant capital
hde_id_ex(t,r,ss,s)$(extant_share(r,s) and id0(r,ss,s))..
  id_ex(t,r,ss,s) =e= prod_ind(t,r,ss,s,"extant")*y_ex(t,r,s);

* hicksian demand equation - fixed factor resources with extant capital
hde_res_ex(t,r,s)$(extant_share(r,s) and res0(r,s))..
  res_ex(t,r,s) =e= y_ex(t,r,s);

* hicksian demand equation - abatement with extant capital
hde_abate_ex(t,r,s,"extant")$(abate0(t,r,s,"extant"))..
  abate(t,r,s,"extant") =e= y_ex(t,r,s);

* hicksian demand equation - abatement with new capital
hde_abate(t,r,s,"new")$(abate0(t,r,s,"new"))..
  abate(t,r,s,"new") =e= y(t,r,s);

* hicksian demand equation - intermediates for abatement
hde_id_abate(t,r,ss,s,v)$(id_abate0(t,r,ss,s,v))..
  id_abate(t,r,ss,s,v) =e= abate_ind(t,r,ss,s)*abate(t,r,s,v);

* hicksian demand equation - labor for abatement
hde_ld_abate(t,r,s,v)$(ld_abate0(t,r,s,v))..
  ld_abate(t,r,s,v) =e= abate_ind(t,r,"l",s)*abate(t,r,s,v);

* hicksian demand equation - capital for abatement
hde_kd_abate(t,r,s,v)$(kd_abate0(t,r,s,v))..
  kd_abate(t,r,s,v) =e= abate_ind(t,r,"k",s)*abate(t,r,s,v);

* hicksian demand equation - resource(r)-capital(k)-labor(l)-energy(e)-
* materials(m) bundle
hde_rklem(t,r,s)$(y0(r,s))..
  rklem(t,r,s) =e= y(t,r,s);

* hicksian demand equation - fixed factor resource
hde_res(t,r,s)$(res0(r,s))..
  res(t,r,s) =e= y(t,r,s)*( uc_y(t,r,s)/(1-ty0(r,s))
                           *(1+tk0(r))/((1+tk(t,r))*pres(t,r,s))
                          )**se_rklem(t,r,s);

* hicksian demand equation - capital(k)-labor(l)-energy(e)-materials(m) bundle
hde_klem(t,r,s)$(y0(r,s))..
  klem(t,r,s) =e= y(t,r,s)*(uc_y(t,r,s)/(1-ty0(r,s))/pklem(t,r,s))
                           **se_rklem(t,r,s);

* hicksian demand equation - capital(k)-labor(l)-energy(e) bundle
hde_kle(t,r,s)$(y0(r,s))..
  kle(t,r,s) =e= klem(t,r,s)*(uc_klem(t,r,s)/pkle(t,r,s))**se_klem(s);

* hicksian demand equation - capital(k)-labor(l) bundle
hde_kl(t,r,s)$(y0(r,s))..
  kl(t,r,s) =e= kle(t,r,s)*(uc_kle(t,r,s)/pkl(t,r,s))**se_kle(s);

* hicksian demand equation - labor for use with new capital
hde_ld(t,r,s)$(y0(r,s))..
  ld(t,r,s) =e= kl(t,r,s)*prod_ind(t,r,"l",s,"new")**(1-se_kl(s))
                *(uc_kl(t,r,s)*(1+tp0(r))/((1+tp(t,r))*pl(t,r)))**se_kl(s);

* hicksian demand equation - new capital
hde_kd(t,r,s)$(y0(r,s))..
  kd(t,r,s) =e= kl(t,r,s)*prod_ind(t,r,"k",s,"new")**(1-se_kl(s))
                *(uc_kl(t,r,s)*(1+tk0(r))*pr0/((1+tk(t,r))*pr(t,r)))**se_kl(s);

* hicksian demand equation - primary energy(en)-electricity(e) bundle
hde_ene(t,r,s)$(y0(r,s))..
  ene(t,r,s) =e= kle(t,r,s)*(uc_kle(t,r,s)/pene(t,r,s))**se_kle(s);

* hicksian demand equation - electricity intermediates
hde_id_ele(t,r,sel,s)$(y0(r,s))..
  id(t,r,sel,s) =e= ene(t,r,s)*prod_ind(t,r,sel,s,"new")**(1-se_ene(s))
                      *(uc_ene(t,r,s)/pa(t,r,sel))**se_ene(s);

* hicksian demand equation - primary energy(en) bundle
hde_en(t,r,s)$(y0(r,s))..
  en(t,r,s) =e= ene(t,r,s)*(uc_ene(t,r,s)/pen(t,r,s))**se_ene(s);

* hicksian demand equation - materials(m) bundle
hde_mat(t,r,s)$(y0(r,s))..
  mat(t,r,s) =e= klem(t,r,s)*(uc_klem(t,r,s)/pmat(t,r,s))**se_klem(s);

* hicksian demand equation - materials intermediates
hde_id_m(t,r,sm,s)$(y0(r,s))..
  id(t,r,sm,s) =e= prod_ind(t,r,sm,s,"new")*mat(t,r,s);

* hicksian demand equation - primary energy
hde_id_en(t,r,sen,s)$(y0(r,s))..
  id(t,r,sen,s) =e= en(t,r,s)*prod_ind(t,r,sen,s,"new")**(1-se_en(s))
                    *(uc_en(t,r,s)/pa(t,r,sen))**se_en(s);

* hicksian demand equation - domestic-national armington aggregate
hde_dn(t,r,s)..
  dn(t,r,s) =e= a(t,r,s)*(uc_a(t,r,s)/pdn(t,r,s))**se_nf(s);

* hicksian demand equation - imports
hde_m(t,r,s,trd)$(m0(r,s,trd))..
  m(t,r,s,trd) =e=  (dn(t,r,s)*(uc_dn(t,r,s)/pn(t,s))**se_dn(s))$(dtrd(trd))
                   +(a(t,r,s)*(uc_a(t,r,s)/(pm(t,s)))**se_nf(s))$(ftrd(trd));

* hicksian demand equation - domestic supply
hde_d(t,r,s)$(d0(r,s))..
  d(t,r,s) =e= dn(t,r,s)*(uc_dn(t,r,s)/pd(t,r,s))**se_dn(s);

* hicksian demand equation - exports
hde_x(t,r,s,trd)$(x0(r,s,trd))..
  x(t,r,s,trd) =e= (y_ex(t,r,s)+y(t,r,s))
                   *( ((pn(t,s)/py(t,r,s))**te_dx(s))$(dtrd(trd))
                     +((px(t,s)/py(t,r,s))**te_dx(s))$(ftrd(trd)));

* hicksian demand equation - leisure
hde_leis(t,r,h)..
  leis(t,r,h) =e= cl(t,r,h)*pref_ind(t,r,"l",h)**(1-se_cl(r,h))
                    *(uc_cl(t,r,h)*(1-tl0(r,h)-tfica0(r,h))
                        /((1-tl(t,r,h)-tfica(t,r,h))*pl(t,r)))**se_cl(r,h);

* hicksian demand equation - consumption bundle
hde_dc(t,r,h)..
  dc(t,r,h) =e= cl(t,r,h)*(uc_cl(t,r,h)/pc(t,r,h))**se_cl(r,h);

* hicksian demand equation - aggregate consumption bundle
hde_c(t,r,h)..
  c(t,r,h) =e= (dc(t,r,h)*dc0(r,h) + sum(s, scd(t,r,s,h)*scd_base(t,r,s,h)))/
               (dc0(r,h) + sum(s, scd_base(t,r,s,h)));

* hicksian demand equation - consumption of other commodities
hde_dcd(t,r,s,h)..
  dcd(t,r,s,h) =e= dc(t,r,h)*(uc_c(t,r,h)/((1+tc(t,r,s))*pa(t,r,s)));

* hicksian demand equation - non-discretionary consumption materials
hde_scd(t,r,s,h)..
  scd(t,r,s,h) =e= pref_ind(t,r,s,h)*q_base(t);

* hicksian demand equation - total commodity demand
hde_cd(t,r,s,h)$cd_base(t,r,s,h)..
  cd(t,r,s,h) =e= (dcd(t,r,s,h)*dcd_base(t,r,s,h) + scd(t,r,s,h)*scd_base(t,r,s,h))/
                  (dcd_base(t,r,s,h) + scd_base(t,r,s,h));

* hicksian demand equation - investment
hde_i(t,r,s)..
  i(t,r,s) =e= inv(t,r)*(uc_inv(t,r)/pa(t,r,s))**se_inv;

* hicksian demand equation - government
hde_g(t,r,s)..
  g(t,r,s) =e= gov(t,r);

* hicksian demand equation - open economy export demand input
hde_xd(t,s)$(sum((r,ftrd), x0(r,s,ftrd)))..
  px(t,s)*xd(t,s) =e= xd_fx(t,s)*uc_loe_x(t,s);

* hicksian demand equation - open economy export fixed factor
hde_fix(t,s)$fix0(s)..
   pfix(t,s)*fix(t,s) =e= xd_fx(t,s)*uc_loe_x(t,s);

* hicksian demand equation - open economy import pfx input
hde_ms_fx(t,s)$(sum((r,ftrd), m0(r,s,ftrd)))..
  pfx(t)*ms_fx(t,s) =e= ms(t,s)*uc_loe_m(t,s);

* hicksian demand equation - open economy import fixed factor input
hde_fim(t,s)$fim0(s)..
  pfim(t,s)*fim(t,s) =e= ms(t,s)*uc_loe_m(t,s);



* ------------------------------------------------------------------------------
* household first order conditions
* ------------------------------------------------------------------------------

* first order condition - consumption(c)-leisure(l) bundle
foc_cl(t,r,h)..
  lambda(t,r,h)*pcl(t,r,h)*(cl(t,r,h)*cl0(r,h)/n(t,r,h))**(eta(r,h)) =e= 1;

* first order condition - savings
foc_lambda(t+1,r,h)..
  beta(t+1,r,h)*lambda(t+1,r,h) =e= beta(t,r,h)*lambda(t,r,h);



* ------------------------------------------------------------------------------
* zero profit conditions
* ------------------------------------------------------------------------------

* zero profit - output with extant capital
zp_y_ex(t,r,s)$(extant_share(r,s) and y0(r,s))..
  uc_y_ex(t,r,s) =e= py(t,r,s)*(1-ty(t,r,s));

* zero profit - output with new capital
zp_y(t,r,s)$(y0(r,s))..
  uc_y(t,r,s) =e= py(t,r,s)*(1-ty(t,r,s));

* zero profit - abatement
zp_abate(t,r,s,v)$(abate0(t,r,s,v))..
  uc_abate(t,r,s,v) =e= pabate(t,r,s,v);

* zero profit - resource(r)-capital(k)-labor(l)-energy(e)-materials(m) bundle
zp_rklem(t,r,s)$(y0(r,s))..
  uc_rklem(t,r,s) =e= prklem(t,r,s);

* zero profit - capital(k)-labor(l)-energy(e)-materials(m) bundle
zp_klem(t,r,s)$(y0(r,s))..
  uc_klem(t,r,s) =e= pklem(t,r,s);

* zero profit - capital(k)-labor(l)-energy(e) bundle
zp_kle(t,r,s)$(y0(r,s))..
  uc_kle(t,r,s) =e= pkle(t,r,s);

* zero profit - capital(k)-labor(l) bundle
zp_kl(t,r,s)$(y0(r,s))..
  uc_kl(t,r,s) =e= pkl(t,r,s);

* zero profit - primary energy(en)-electricity(e) bundle
zp_ene(t,r,s)$(y0(r,s))..
  uc_ene(t,r,s) =e= pene(t,r,s);

* zero profit - primary energy(en) bundle
zp_en(t,r,s)$(y0(r,s))..
  uc_en(t,r,s) =e= pen(t,r,s);

* zero profit - materials(m) bundle
zp_mat(t,r,s)$(y0(r,s))..
  uc_mat(t,r,s) =e= pmat(t,r,s);

* zero profit - armington aggregate
zp_a(t,r,s)$(a0(r,s))..
  uc_a(t,r,s) =e= pa(t,r,s);

* zero profit - domestic-national armington aggregate
zp_dn(t,r,s)$(dn0(r,s))..
  uc_dn(t,r,s) =e= pdn(t,r,s);

* zero profit - consumption-leisure bundle
zp_cl(t,r,h)$(cl0(r,h))..
  uc_cl(t,r,h) =e= pcl(t,r,h);

* zero profit - consumption bundle
zp_c(t,r,h)$(dc0(r,h))..
  uc_c(t,r,h) =e= pc(t,r,h);

* zero profit - investment
zp_inv(t,r)$(inv0(r))..
  uc_inv(t,r) =e= pk(t+1,r)+pkt(r)$tlast(t);

* zero profit - capital stock
zp_k(t,r)..
  pk(t,r) =e= interval(t)*pr(t,r)
              +(pk(t+1,r)+pkt(r)$tlast(t))*(1-delta)**interval(t);

* zero profit - household capital stock
zp_pkh(t)..
  pkh(t) =e= interval(t)*prh(t)+(pkh(t+1)+pkht$tlast(t))*(1-delta)**interval(t);

* zero profit - household terminal capital stock
zp_pkht..
  pkht*sum(r, kt(r)*k0(r)) =e= sum(r, pkt(r)*kt(r)*k0(r));

* zero profit - government bundle
zp_gov(t,r)$(gov0(r))..
  uc_gov(t,r) =e= pgov(t,r);

* zero profit - open economy exports
zp_xd_fx(t,s)$(sum((r,ftrd), x0(r,s,ftrd)))..
  uc_loe_x(t,s) =e= pfx(t);

* zero profit - open economy imports
zp_ms(t,s)$(sum((r,ftrd), m0(r,s,ftrd)))..
  uc_loe_m(t,s) =e= pm(t,s);



* ------------------------------------------------------------------------------
* market clearance conditions
* ------------------------------------------------------------------------------

* market clearance - domestic output
mc_d(t,r,s)$(y0(r,s))..
  (y_ex(t,r,s)+y(t,r,s))*(pd(t,r,s)/py(t,r,s))**te_dx(s) =e= d(t,r,s);

* market clearance - goods market
mc_a(t,r,s)$(a0(r,s))..
  a(t,r,s)*a0(r,s) =e=  sum(ss, id(t,r,s,ss)*id_base(t,r,s,ss))
                       +sum(ss, id_ex(t,r,s,ss)*id0(r,s,ss))
                       +sum((ss,v), id_abate(t,r,s,ss,v)*id_abate0(t,r,s,ss,v))
                       +sum(h, cd(t,r,s,h)*cd_base(t,r,s,h))
                       +i(t,r,s)*i0(r,s)+g(t,r,s)*g_base(t,r,s);

* market clearance - labor
mc_l(t,r)..
  sum(h, l(t,r,h)*l0(r,h)) =e=  sum(s, ld_ex(t,r,s)*ld0(r,s))
                               +sum(s, ld(t,r,s)*ld_base(t,r,s))
                               +sum((s,v), ld_abate(t,r,s,v)
                                          *ld_abate0(t,r,s,v));

* market clearance - time
mc_time(t,r,h)..
 te(t,r,h) =e= leis(t,r,h)*leis0(r,h)+l(t,r,h)*l0(r,h);

* market clearance - rental rate for sector differentiated extant capital
mc_pr_ex(t,r,s)$(extant_share(r,s) and y0(r,s))..
  k_ex(t,r)*extant_share(r,s)/extant_share_agg(r)
  *(pr_ex(t,r,s)/pr_ex_agg(t,r))**te_k_ex
    =e= kd_ex(t,r,s);

* market clearance - rental rate for new capital
mc_pr(t,r)..
  k(t,r)*k0(r) =e=  sum(s, kd(t,r,s)*kd_base(t,r,s))
                   +sum((s,v), kd_abate(t,r,s,v)*kd_abate0(t,r,s,v));

* market clearance - price of new capital
mc_pk(t,r)..
  k(t-1,r)*k0(r)*(1-delta)**interval(t-1)+interval(t-1)*inv(t-1,r)*inv0(r)
  +((1-extant_share_agg(r))*k0(r))$tfirst(t)
    =e= k(t,r)*k0(r);

* market clearance - household investment
mc_invh(t,r,h)..
  (1-delta)**interval(t)*kh(t,r,h)*ke0(r,h)+interval(t)*invh(t,r,h)*invh0(r,h)
    =e= (kh(t+1,r,h)+kht(r,h)$tlast(t))*ke0(r,h);

* market clearance - household new capital
mc_prh(t)..
  sum((r,h), kh(t,r,h)*ke0(r,h))$(not tfirst(t))
  +prh(t)*sum((r), k(t,r)*k0(r))$tfirst(t)
    =e= sum(r, k(t,r)*k0(r))$(not tfirst(t))
        +sum(r, pr(t,r)*k(t,r)*k0(r))$tfirst(t);

* market clearnace - open economy exports
mc_px(t,s)$(sum((r,ftrd), x0(r,s,ftrd)))..
  sum((r,ftrd), x0(r,s,ftrd)*x(t,r,s,ftrd))
    =e= sum((r,ftrd), x0(r,s,ftrd))*xd(t,s);

* market clearance - open economy exports (fixed factor)
mc_pfix(t,s)$(fix0(s))..
  fix0(s)*q_base(t) =g= fix0(s)*fix(t,s);

* market clearance - open economy imports
mc_pm(t,s)$(sum((r,ftrd), m0(r,s,ftrd)))..
  sum((r,ftrd), m0(r,s,ftrd))*ms(t,s)
    =e= sum((r,ftrd), m0(r,s,ftrd)*m(t,r,s,ftrd));

* market clearance - open economy imports (fixed factor)
mc_pfim(t,s)$(fim0(s))..
  fim0(s)*q_base(t) =g= fim0(s)*fim(t,s);

* market clearance - foreign trade
mc_fx(t)..
  sum(s, (fix0(s) + sum((r,ftrd), x0(r,s,ftrd)))*xd_fx(t,s))
  +inc_row0*inc_row(t)+tran_row0*tran_row(t)-gint_row0*gint_row(t)
  -curactbal0*curactbal(t)
    =e= sum(s, (sum((r,ftrd), m0(r,s,ftrd))-fim0(s))*ms_fx(t,s))+row(t);

* market clearance - national trade
mc_dtrd(t,s)..
  sum((r,dtrd), x(t,r,s,dtrd)*x0(r,s,dtrd))
    =e= sum((r,dtrd), m(t,r,s,dtrd)*m0(r,s,dtrd));

* market clearance - price of post-terminal capital
mc_pkt(r)..
  sum(t$tlast(t), k(t,r)*k0(r)*(1-delta)**interval(t)
                 +interval(t)*inv(t,r)*inv0(r)) =e= kt(r)*k0(r);

* market clearnace - fixed factor resources
mc_pres(t,r,s)$res0(r,s)..
*  res0(r,s)*(pres(t,r,s)/cpi(t))**re(s)
*    =g= (res_ex(t,r,s)+res(t,r,s))*res0(r,s);
  res0(r,s) =g= (res_ex(t,r,s)+res(t,r,s))*res0(r,s);



* ------------------------------------------------------------------------------
* budget constraints
* ------------------------------------------------------------------------------

* budget constraint - household
bc_hh(t,r,h)..
  (pkh(t+1)+pkht$tlast(t))*invh(t,r,h)*invh0(r,h)$(card(t) gt 1)
  +pkt(r)*invh(t,r,h)*invh0(r,h)$(card(t) eq 1)
    =e= prh(t)*kh(t,r,h)*ke0(r,h)$(card(t) gt 1)
        +pr(t,r)*kh(t,r,h)*ke0(r,h)$(card(t) eq 1)
        +(1-tl(t,r,h)-tfica(t,r,h))*pl(t,r)*te(t,r,h)
        +(tl(t,r,h)-tl_avg0(r,h))*pl(t,r)*l(t,r,h)*l0(r,h)
        +prh_ex(t)*kh_ex(t,r,h)*ke0(r,h)
        +sum(s, pres(t,r,s)*(res_ex(t,r,s)+res(t,r,s))*rese0(r,s,h))
        +cpi(t)*oth_adj0(r,h)*oth_adj(t)
        +cpi(t)*incadj(t)*incadj_share(r,h)
        +cpi(t)*(tran_fed0(r,h)+tran_state0(r,h))*transfers(t)
        +cpi(t)*gint0*gint(t)*g0_share(r,h)
        -cpi(t)*deficit0*deficit(t)*g0_share(r,h)
        +pfx(t)*inc_row0*inc_row(t)*int0_share(r,h)
        +pfx(t)*tran_row0*tran_row(t)*int0_share(r,h)
        -pfx(t)*curactbal0*curactbal(t)*int0_share(r,h)
        +inc_extra(t,r,h)
  	    -sum(s, pa(t,r,s)*(1+tc(t,r,s))*scd(t,r,s,h)*scd_base(t,r,s,h))
        -pcl(t,r,h)*cl(t,r,h)*cl0(r,h);

* budget constraint - government
bc_gov(t)..
  sum(r, pgov(t,r)*gov(t,r)*gov0(r))
  +cpi(t)*sum((r,h), tran_fed0(r,h)+tran_state0(r,h))*transfers(t)
  +cpi(t)*gint0*gint(t)
  +pfx(t)*gint_row0*gint_row(t)
  +cpi(t)*sum((r,h), oth_adj0(r,h))*oth_adj(t)
  +cpi(t)*incadj(t)
  -cpi(t)*deficit0*deficit(t)
    =e= sum(r, sum(s, tp(t,r)*pl(t,r)*ld(t,r,s)*ld_base(t,r,s)
                     +tp(t,r)*pl(t,r)*ld_ex(t,r,s)*ld0(r,s)
                     +tp(t,r)*pl(t,r)*sum(v, ld_abate(t,r,s,v)
                                            *ld_abate0(t,r,s,v))
                     +tk(t,r)*pr(t,r)*kd(t,r,s)*kd_base(t,r,s)
                     +tk(t,r)*pres(t,r,s)*( res_ex(t,r,s)
                                           +res(t,r,s))*res0(r,s)
                     +tk(t,r)*pr_ex(t,r,s)*kd_ex(t,r,s)*kd0(r,s)
                     +tk(t,r)*pr(t,r)*sum(v, kd_abate(t,r,s,v)
                                            *kd_abate0(t,r,s,v))
                     +ty(t,r,s)*py(t,r,s)*y_ex(t,r,s)*y0(r,s)
                     +ty(t,r,s)*py(t,r,s)*y(t,r,s)*y0(r,s)
                     +sum(h, tc(t,r,s)*pa(t,r,s)*cd(t,r,s,h)*cd_base(t,r,s,h)))
                     +sum(h, tl_avg0(r,h)*pl(t,r)*l(t,r,h)*l0(r,h)
                            +tfica(t,r,h)*pl(t,r)*l(t,r,h)*l0(r,h)));

* budget constraint - rest of world
bc_row(t)$(sum(s, fim0(s)) or sum(s, fix0(s)))..
  pfx(t)*row(t) =e= sum(s, pfix(t,s)*fix0(s)*fix(t,s)
                          +pfim(t,s)*fim0(s)*fim(t,s));



* ------------------------------------------------------------------------------
* closure rules
* ------------------------------------------------------------------------------

* closure rule - real government expenditures per capita are fixed
cr_gov(t,r)..
  gov(t,r)*gov0(r) =e= gov_exp(t)*gov0(r)+gov_extra(t,r);

* closure rule - terminal capital
* scale the level of the terminal capital stock to achieve steady-state growth
* in last period investment.
cr_kt(r)..
  sum(tlast(t), inv(t,r)*sum(s$y0(r,s), y(t-1,r,s)+y_ex(t-1,r,s)))
    =e= sum(tlast(t), sum(s$y0(r,s), y(t,r,s)+y_ex(t,r,s))*inv(t-1,r));

* closure rule - household terminal capital
* share of aggregate terminal capital stock is based on the last periods share
* of aggregate captial stock for the household
cr_kht(r,h)..
    kht(r,h)*ke0(r,h)/sum(rr, kt(rr)*k0(rr))
      =e= sum(t$tlast(t), kh(t,r,h)*ke0(r,h))
            /sum((t,rr,hh)$tlast(t), kh(t,rr,hh)*ke0(rr,hh));



* ------------------------------------------------------------------------------
* auxilary
* ------------------------------------------------------------------------------

* auxilary - regional extant capital
aux_k_ex(t,r)$(extant_share_agg(r))..
  k_ex(t,r) =e= extant_share_agg(r)*(1-delta)**(t.val-sum(tfirst, tfirst.val));

* auxilary - household extant capital
aux_kh_ex(t,r,h)$(sum(rr, extant_share_agg(rr)))..
  kh_ex(t,r,h) =e= sum(rr, k_ex(t,rr)*k0(rr))/sum(rr, k0(rr));

* auxilary - national extant capital rental rate
aux_prh_ex(t)$(sum(r, extant_share_agg(r)))..
  prh_ex(t) =e= sum(r, pr_ex_agg(t,r)*k_ex(t,r)*k0(r))/sum(r, k_ex(t,r)*k0(r));

* auxilary - welfare
aux_w(r,h)..
  w(r,h) =e= sum(t,  n(t,r,h)*beta(t,r,h)
                    *(cl(t,r,h)*cl0(r,h)/n(t,r,h))**(1-eta(r,h))/(1-eta(r,h)));

* auxilary - consumer price index
aux_cpi(t)..
  cpi(t) =e= sum((rr,hh), pc(t,rr,hh)*c0(rr,hh))/sum((rr,hh), c0(rr,hh));

* auxilary - gdp
aux_gdp(t)..
  gdp(t)*gdp0 =e= sum((r,s,h), pa(t,r,s)*cd(t,r,s,h)*cd_base(t,r,s,h))
                 +sum((r,s), pa(t,r,s)*i(t,r,s)*i0(r,s))
                 +sum((r,s), pa(t,r,s)*g(t,r,s)*g_base(t,r,s))
                 +sum((r,s), px(t,s)*x(t,r,s,"ftrd")*x0(r,s,"ftrd"))
                 -sum((r,s), pm(t,s)*m(t,r,s,"ftrd")*m0(r,s,"ftrd"));



* ------------------------------------------------------------------------------
* define mcp problem
* ------------------------------------------------------------------------------

model sage /

* zero profit conditions - determine demand for bundled commodities
  zp_y_ex.y_ex
  zp_y.y
  zp_abate.abate
  zp_rklem.rklem
  zp_klem.klem
  zp_kle.kle
  zp_kl.kl
  zp_ene.ene
  zp_en.en
  zp_mat.mat
  zp_a.a
  zp_dn.dn
  zp_cl.pcl
  zp_c.dc
  zp_inv.inv
  zp_k.k
  zp_pkh.pkh
  zp_pkht.pkht
  zp_gov.gov
  zp_xd_fx.xd_fx
  zp_ms.ms

* market clearance conditions - determine prices for underlying commodities
  mc_pr_ex.pr_ex
  mc_pres.pres
  mc_d.pd
  mc_a.pa
  mc_pr.pr
  mc_pk.pk
  mc_invh.invh
  mc_prh.prh
  mc_fx.pfx
  mc_pkt.pkt
  mc_time.l
  mc_l.pl
  mc_dtrd.pn
  mc_px.px
  mc_pm.pm
  mc_pfix.pfix
  mc_pfim.pfim

* hicksian demand equations - determine prices for bundled commodities
  hde_abate_ex.pabate
  hde_abate.pabate
  hde_rklem.prklem
  hde_klem.pklem
  hde_kle.pkle
  hde_kl.pkl
  hde_ene.pene
  hde_en.pen
  hde_mat.pmat
  hde_dn.pdn
  hde_dc.pc

* hicksian demand equations - determine demand for underlying commodities
  hde_ld_ex.ld_ex
  hde_kd_ex.kd_ex
  hde_id_ex.id_ex
  hde_res_ex.res_ex
  hde_id_abate.id_abate
  hde_ld_abate.ld_abate
  hde_kd_abate.kd_abate
  hde_res.res
  hde_ld.ld
  hde_kd.kd
  hde_id_ele.id
  hde_id_m.id
  hde_id_en.id
  hde_m.m
  hde_d.d
  hde_x.x
  hde_leis.leis
  hde_c.c
  hde_cd.cd
  hde_dcd.dcd
  hde_scd.scd
  hde_i.i
  hde_g.g
  hde_xd.xd
  hde_fix.fix
  hde_fim.fim
  hde_ms_fx.ms_fx

* budget constraints
  bc_hh.lambda
  bc_gov.incadj
  bc_row.row

* first order conditions
  foc_cl.cl
  foc_lambda.kh

* unit cost equations
  uce_y_ex.uc_y_ex
  uce_y.uc_y
  uce_abate.uc_abate
  uce_rklem.uc_rklem
  uce_klem.uc_klem
  uce_kle.uc_kle
  uce_kl.uc_kl
  uce_ene.uc_ene
  uce_en.uc_en
  uce_mat.uc_mat
  uce_a.uc_a
  uce_dn.uc_dn
  uce_dx.py
  uce_k_ex.pr_ex_agg
  uce_cl.uc_cl
  uce_c.uc_c
  uce_inv.uc_inv
  uce_gov.uc_gov
  uce_loe_x.uc_loe_x
  uce_loe_m.uc_loe_m

* closure rules
  cr_gov.pgov
  cr_kt.kt
  cr_kht.kht

* auxilary equations
  aux_k_ex.k_ex
  aux_kh_ex.kh_ex
  aux_prh_ex.prh_ex
  aux_w.w
  aux_cpi.cpi
  aux_gdp.gdp

/;



* ------------------------------------------------------------------------------
* scale the model
* ------------------------------------------------------------------------------

bc_hh.scale(t,r,h)           = 1e4;
zp_pkht.scale                = 1e5;
aux_gdp.scale(t)             = 1e4;
mc_invh.scale(t,r,h)         = 1e3;
mc_a.scale(t,r,s)            = 1e3;
zp_k.scale(t,r)              = 1e3;
bc_gov.scale(t)              = 1e4;
mc_pk.scale(t,r)             = 1e4;
mc_prh.scale(t)              = 1e4;
mc_pkt.scale(r)              = 1e4;
mc_l.scale(t,r)              = 1e3;
mc_pr_ex.scale(t,r,s)        = 1e1;
mc_pr.scale(t,r)             = 1e4;
mc_time.scale(t,r,h)         = 1e3;
mc_fx.scale(t)               = 1e3;
cr_gov.scale(t,r)            = 1e4;
hde_d.scale(t,r,s)           = 1e3;
hde_dn.scale(t,r,s)          = 1e2;
hde_m.scale(t,r,s,trd)       = 1e3;
hde_x.scale(t,r,s,trd)       = 1e2;
hde_res.scale(t,r,s)         = 1e1;
hde_kd.scale(t,r,s)          = 1e2;
hde_en.scale(t,r,s)          = 1e2;
hde_kl.scale(t,r,s)          = 1e1;
hde_kle.scale(t,r,s)         = 1e1;
hde_mat.scale(t,r,s)         = 1e1;
hde_ld.scale(t,r,s)          = 1e1;
hde_ene.scale(t,r,s)         = 1e2;
hde_id_ele.scale(t,r,sel,s)  = 1e1;
hde_klem.scale(t,r,s)        = 1e1;
hde_id_en.scale(t,r,sen,s)   = 1e1;
mc_dtrd.scale(t,s)           = 1e3;
mc_d.scale(t,r,s)            = 1e2;
hde_leis.scale(t,r,h)        = 1e4;
hde_dc.scale(t,r,h)          = 1e2;
hde_c.scale(t,r,h)           = 1e2;
hde_cd.scale(t,r,s,h)        = 1e1;
hde_dcd.scale(t,r,s,h)       = 1e1;
hde_scd.scale(t,r,s,h)       = 1e1;
foc_cl.scale(t,r,h)          = 1e2;
cr_kt.scale(r)               = 1e1;
zp_xd_fx.scale(t,s)          = 1e1;
zp_ms.scale(t,s)             = 1e1;
mc_px.scale(t,s)             = 1e1;
mc_pm.scale(t,s)             = 1e1;
mc_pfix.scale(t,s)           = 1e1;
mc_pfim.scale(t,s)           = 1e1;
hde_xd.scale(t,s)            = 1e1;
hde_fix.scale(t,s)           = 1e1;
hde_fim.scale(t,s)           = 1e1;
hde_ms_fx.scale(t,s)         = 1e1;
bc_row.scale(t)              = 1e4;
uce_loe_x.scale(t,s)         = 1e1;
uce_loe_m.scale(t,s)         = 1e1;

incadj.scale(t) = 1e3;


* ------------------------------------------------------------------------------
* set solver options
* ------------------------------------------------------------------------------

$label solver_options

* use the path options file
sage.optfile = 1;

* use the modeling scaling
sage.scaleopt = 1;

* iteration limit
sage.iterlim = 1000000;

* time limit
sage.reslim = 2*60*60;

* treat fixed variables as constants <- helps reduce the size of the problem
* when only extant capital in the first time period
sage.holdfixed = 1;

* set the solver to path
option mcp=path;

* open the path options file for writing
file path_opt /path.opt/;
put path_opt;

* write the path options
put /"convergence_tolerance 1e-6;"/;
put /"output_final_scaling_statistics yes;"/;

* maximum number of gradient steps allowed before restarting
put /"gradient_step_limit 10;"/;

* options for line searching, watchdoging, and non-monotone descent
put /"nms_initial_reference_factor 10;"/;
put /"nms_maximum_watchdogs 5;"/;
put /"nms_memory_size 2;"/;
put /"nms_mstep_frequency 10;"/;
put /"nms_searchtype line;"/;

put /"proximal_perturbation 0;"/;
put /"lemke_start_type slack;"/;

* turning off the crash phase is usually the fastest way to solve the model
put /"crash_method none;"/;
put /"crash_perturb no;"/;

* close the path options file
putclose;


* if balanced growth path starting values are requested use them
$if %balanced_start_values%==1 $goto set_using_balanced

* if a baseline file is provided use that for the starting values
$if not %gdx_baseline_file%==0 $goto set_with_saved_values



* ------------------------------------------------------------------------------
* set the starting values for the endogenous variables
* ------------------------------------------------------------------------------

$label set_using_balanced

* parameters for starting values
parameters
  k_ext_base(t,r,s)     extant capital path
  k_new_base(t,r,s)     new capital path
  ex_share(t,r,s)       extant capital share;

* extant capital path
k_ext_base(t,r,s)$extant_share(r,s) = extant_share(r,s)
                                      *(1-delta)**(t.val-sum(tfirst, tfirst.val));

* new capital path
k_new_base(t,r,s) = q_base(t)
                    -extant_share(r,s)*(1-delta)**(t.val-sum(tfirst, tfirst.val));

* extant capital share
ex_share(t,r,s) = k_ext_base(t,r,s)/(k_ext_base(t,r,s)+k_new_base(t,r,s));

* prices
py.l(t,r,s)          = p_base(t);
pabate.l(t,r,s,v)    = p_base(t);
pres.l(t,r,s)        = 1;
prklem.l(t,r,s)      = p_base(t);
pklem.l(t,r,s)       = p_base(t);
pkle.l(t,r,s)        = p_base(t);
pkl.l(t,r,s)         = p_base(t);
pene.l(t,r,s)        = p_base(t);
pen.l(t,r,s)         = p_base(t);
pmat.l(t,r,s)        = p_base(t);
pa.l(t,r,s)          = p_base(t);
pdn.l(t,r,s)         = p_base(t);
pd.l(t,r,s)          = p_base(t);
pn.l(t,s)            = p_base(t);
pcl.l(t,r,h)         = p_base(t)*(cs_cl(r,h,"c")+cs_cl(r,h,"leis")*l_prod_agg(t));
pc.l(t,r,h)          = p_base(t);
pgov.l(t,r)          = p_base(t);
pfx.l(t)             = p_base(t);
pl.l(t,r)            = p_base(t)*l_prod_agg(t);
pk.l(t,r)            = p_base(t)*(1+rbar);
pkh.l(t)             = p_base(t)*(1+rbar);
pr.l(t,r)            = pr0*p_base(t);
prh.l(t)             = pr0*p_base(t);
pr_ex.l(t,r,s)       = pr0*p_base(t);
pr_ex_agg.l(t,r)     = pr0*p_base(t);
prh_ex.l(t)          = pr0*p_base(t);
pkt.l(r)             = sum(t$tlast(t), p_base(t));
pkht.l               = sum(t$tlast(t), p_base(t));

* export and import prices
px.l(t,s)            = p_base(t);
pm.l(t,s)            = p_base(t);
pfix.l(t,s)          = p_base(t);
pfim.l(t,s)          = p_base(t);

* rest of world budget
row.l(t)             = sum(s, pfix.l(t,s)*fix0(s)*q_base(t) +
                              pfim.l(t,s)*fim0(s)*q_base(t))/p_base(t);

* shadow prices
lambda.l(t,r,h)      = (cl0(r,h)
                        /sum(tfirst, n(tfirst,r,h)))**(-eta(r,h))/beta(t,r,h);

* activities
y_ex.l(t,r,s)        = k_ext_base(t,r,s);
y.l(t,r,s)           = k_new_base(t,r,s);
a.l(t,r,s)           = q_base(t);
inv.l(t,r)           = q_base(t);
invh.l(t,r,h)        = q_base(t);
gov.l(t,r)           = gov_exp(t);
l.l(t,r,h)           = n(t,r,h)/n0(r,h);

* demands
ld_ex.l(t,r,s)       = l_prod_agg(t)*k_ext_base(t,r,s);
kd_ex.l(t,r,s)       = k_ext_base(t,r,s);
id_ex.l(t,r,ss,s)    = k_ext_base(t,r,s);
res_ex.l(t,r,s)      = ex_share(t,r,s);
rklem.l(t,r,s)       = k_new_base(t,r,s);
klem.l(t,r,s)        = k_new_base(t,r,s);
res.l(t,r,s)         = (1-ex_share(t,r,s));
kl.l(t,r,s)          = k_new_base(t,r,s);
ld.l(t,r,s)          = sum(h, n(t,r,h))/sum(h, n0(r,h))-ld_ex.l(t,r,s);
kd.l(t,r,s)          = k_new_base(t,r,s);
mat.l(t,r,s)         = k_new_base(t,r,s);
kle.l(t,r,s)         = k_new_base(t,r,s);
ene.l(t,r,s)         = k_new_base(t,r,s);
en.l(t,r,s)          = k_new_base(t,r,s);
id.l(t,r,ss,s)       = k_new_base(t,r,s);
d.l(t,r,s)           = q_base(t);
dn.l(t,r,s)          = q_base(t);
cl.l(t,r,h)          = q_base(t);
leis.l(t,r,h)        = n(t,r,h)/n0(r,h);
c.l(t,r,h)           = q_base(t);
dc.l(t,r,h)          = q_base(t);
cd.l(t,r,s,h)        = q_base(t);
dcd.l(t,r,s,h)       = q_base(t);
scd.l(t,r,s,h)       = q_base(t);
i.l(t,r,s)           = q_base(t);
g.l(t,r,s)           = q_base(t);

parameter
  xm_index_start(t)     index for trade starting values;

xm_index_start(t) = (inc_row0*inc_row(t)+tran_row0*tran_row(t)
                     +curactbal0*curactbal(t))/(inc_row0+tran_row0+curactbal0);

m.l(t,r,s,trd)       = xm_index_start(t);
x.l(t,r,s,trd)       = xm_index_start(t);
ms_fx.l(t,s)         = xm_index_start(t);
xd_fx.l(t,s)         = xm_index_start(t);
ms.l(t,s)            = xm_index_start(t);
xd.l(t,s)            = xm_index_start(t);
fim.l(t,s)           = q_base(t);
fix.l(t,s)           = q_base(t);

* stocks
k_ex.l(t,r)          = extant_share_agg(r)*(1-delta)**(t.val-sum(tfirst, tfirst.val));
kh_ex.l(t,r,h)       = k_ex.l(t,r);
k.l(t,r)             = q_base(t)-k_ex.l(t,r);
kh.l(t,r,h)          = k.l(t,r);
kt.l(r)              = sum(t$tlast(t), l_prod_agg(t)*l_force(t))
                       -extant_share_agg(r)*(1-delta)**(sum(tlast, tlast.val)
                                                        -sum(tfirst, tfirst.val)
                                                        +sum(tlast, interval(tlast)));
kht.l(r,h)           = kt.l(r);

* unit costs
uc_y_ex.l(t,r,s)     = (1-ty0(r,s))*p_base(t);
uc_y.l(t,r,s)        = (1-ty0(r,s))*p_base(t);
uc_abate.l(t,r,s,v)  = p_base(t);
uc_rklem.l(t,r,s)    = p_base(t);
uc_klem.l(t,r,s)     = p_base(t);
uc_kl.l(t,r,s)       = p_base(t);
uc_mat.l(t,r,s)      = p_base(t);
uc_kle.l(t,r,s)      = p_base(t);
uc_ene.l(t,r,s)      = p_base(t);
uc_en.l(t,r,s)       = p_base(t);
uc_a.l(t,r,s)        = p_base(t);
uc_dn.l(t,r,s)       = p_base(t);
uc_cl.l(t,r,h)       = p_base(t);
uc_c.l(t,r,h)        = p_base(t);
uc_inv.l(t,r)        = p_base(t);
uc_gov.l(t,r)        = p_base(t);
uc_loe_x.l(t,s)      = p_base(t);
uc_loe_m.l(t,s)      = p_base(t);

* auxillary
incadj.l(t)          = 0;
w.l(r,h)             = sum(t, n(t,r,h)*beta(t,r,h)
                             *(cl.l(t,r,h)*cl0(r,h)/n(t,r,h)
                              )**(1-eta(r,h))/(1-eta(r,h)));
cpi.l(t)             = p_base(t);
gdp.l(t)             = p_base(t)*q_base(t);

* abatement
abate.l(t,r,s,"extant")       = k_ext_base(t,r,s);
abate.l(t,r,s,"new")          = k_new_base(t,r,s);
id_abate.l(t,r,ss,s,"extant") = k_ext_base(t,r,s);
id_abate.l(t,r,ss,s,"new")    = k_new_base(t,r,s);
ld_abate.l(t,r,s,"extant")    = k_ext_base(t,r,s);
ld_abate.l(t,r,s,"new")       = k_new_base(t,r,s);
kd_abate.l(t,r,s,"extant")    = k_ext_base(t,r,s);
kd_abate.l(t,r,s,"new")       = k_new_base(t,r,s);

* abatement
abate.l(t,r,s,"extant")       = k_ext_base(t,r,s);
abate.l(t,r,s,"new")          = k_new_base(t,r,s);
id_abate.l(t,r,ss,s,"extant") = k_ext_base(t,r,s);
id_abate.l(t,r,ss,s,"new")    = k_new_base(t,r,s);
ld_abate.l(t,r,s,"extant")    = k_ext_base(t,r,s);
ld_abate.l(t,r,s,"new")       = k_new_base(t,r,s);
kd_abate.l(t,r,s,"extant")    = k_ext_base(t,r,s);
kd_abate.l(t,r,s,"new")       = k_new_base(t,r,s);

$goto end_starting_values



* ------------------------------------------------------------------------------
* set the starting values for the endogenous variables using baseline results
* ------------------------------------------------------------------------------

$label set_with_saved_values

* open the gdx file connection
$gdxin %gdx_baseline_file%

* prices
$loadm py
$loadm pres
$loadm prklem
$loadm pabate
$loadm pklem
$loadm pkle
$loadm pkl
$loadm pene
$loadm pen
$loadm pmat
$loadm pa
$loadm pdn
$loadm pd
$loadm pn
$loadm pcl
$loadm pc
$loadm pgov
$loadm pfx
$loadm pl
$loadm pk
$loadm pkh
$loadm pr
$loadm prh
$loadm pr_ex
$loadm pr_ex_agg
$loadm prh_ex
$loadm pkt
$loadm pkht
$loadm cpi

* large open economy values
$loadm px
$loadm pm
$loadm uc_loe_x
$loadm uc_loe_m
$loadm xd
$loadm xd_fx
$loadm ms
$loadm ms_fx
$loadm row
$loadm pfix
$loadm pfim
$loadm fix
$loadm fim

* shadow prices
$loadm lambda

* activities
$loadm y_ex
$loadm y
$loadm a
$loadm inv
$loadm invh
$loadm gov
$loadm l

* demands
$loadm abate
$loadm ld_abate
$loadm kd_abate
$loadm id_abate
$loadm ld_ex
$loadm kd_ex
$loadm id_ex
$loadm res_ex
$loadm rklem
$loadm klem
$loadm res
$loadm kl
$loadm ld
$loadm kd
$loadm mat
$loadm kle
$loadm ene
$loadm en
$loadm id
$loadm d
$loadm dn
$loadm m
$loadm x
$loadm cl
$loadm leis
$loadm c
$loadm dc
$loadm cd
$loadm dcd
$loadm scd
$loadm i
$loadm g

* stocks
$loadm k_ex
$loadm k
$loadm kh_ex
$loadm kh
$loadm kt
$loadm kht

* unit costs
$loadm uc_y_ex
$loadm uc_y
$loadm uc_abate
$loadm uc_rklem
$loadm uc_klem
$loadm uc_kl
$loadm uc_mat
$loadm uc_kle
$loadm uc_ene
$loadm uc_en
$loadm uc_a
$loadm uc_dn
$loadm uc_cl
$loadm uc_c
$loadm uc_inv
$loadm uc_gov

* auxillary
$loadm incadj
$loadm w

* close the gdx file connection
$gdxin

* ensure starting values for any resources prices are positive
pres.l(t,r,s)$(not pres.l(t,r,s)) = .1;

$label end_starting_values



* ------------------------------------------------------------------------------
* fixed values in most applications and other starting value issues
* ------------------------------------------------------------------------------

* set the numeraire
pfx.fx(t)$tfirst(t) = 1;

* initial household new capital stock is fixed if not all extant in first period
kh.fx(t,r,h)$tfirst(t) = (1-extant_share_agg(r));

* if there is no investment from a household in the sam then fix that at zero
invh.fx(t,r,h)$(not invh0(r,h)) = 0;

* fix variables associated with extant capital if sector has no extant capital
y_ex.fx(t,r,s)$(not extant_share(r,s))      = 0;
ld_ex.fx(t,r,s)$(not extant_share(r,s))     = 0;
kd_ex.fx(t,r,s)$(not extant_share(r,s))     = 0;
id_ex.fx(t,r,ss,s)$(not extant_share(r,s))  = 0;
pr_ex.fx(t,r,s)$(not extant_share(r,s))     = 0;
res_ex.fx(t,r,s)$(not extant_share(r,s))    = 0;
pr_ex_agg.fx(t,r)$(not extant_share_agg(r)) = 0;
prh_ex.fx(t)$(not sum(r, extant_share_agg(r))) = 0;
k_ex.fx(t,r)$(not extant_share_agg(r))      = 0;
kh_ex.fx(t,r,h)$(not sum(rr, extant_share_agg(rr)))   = 0;

* fix first period production at zero for new capital if there is none
y.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)                 = 0;
ld.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)                = 0;
kd.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)                = 0;
id.fx(t,r,ss,s)$(tfirst(t) and extant_share(r,s) eq 1)             = 0;
res.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)               = 0;
rklem.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)             = 0;
klem.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)              = 0;
ene.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)               = 0;
en.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)                = 0;
kl.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)                = 0;
kle.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)               = 0;
mat.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)               = 0;
k.fx(t,r)$(tfirst(t) and extant_share_agg(r) eq 1)                 = 0;
abate.fx(t,r,s,"new")$(tfirst(t) and extant_share(r,s) eq 1)       = 0;
ld_abate.fx(t,r,s,"new")$(tfirst(t) and extant_share(r,s) eq 1)    = 0;
kd_abate.fx(t,r,s,"new")$(tfirst(t) and extant_share(r,s) eq 1)    = 0;
id_abate.fx(t,r,ss,s,"new")$(tfirst(t) and extant_share(r,s) eq 1) = 0;

* if there is no new capital in the first period a number of prices related to
* production with new capital and the value of new capital are not defined so
* we fix those values
pmat.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)            = 1;
pen.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)             = 1;
pene.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)            = 1;
pkl.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)             = 1;
uc_kl.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)           = 1;
pkle.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)            = 1;
pklem.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)           = 1;
prklem.fx(t,r,s)$(tfirst(t) and extant_share(r,s) eq 1)          = 1;
pk.fx(t,r)$(tfirst(t) and extant_share_agg(r) eq 1)              = 1;
pkh.fx(t)$(tfirst(t) and sum(r, extant_share_agg(r)) eq card(r)) = 1;
prh.fx(t)$(tfirst(t) and sum(r, extant_share_agg(r)) eq card(r)) = 1;
pr.fx(t,r)$(tfirst(t) and extant_share_agg(r) eq 1)              = 1;
pabate.fx(t,r,s,"new")$(tfirst(t) and extant_share(r,s) eq 1)    = 1;

* if run as a small open economy there are no fixed factors in the import/export
* supply/demand functions, so their values need to be fixed to satisfy mc_pfix
* and mc_pfim
fix.fx(t,s)$(not fix0(s)) = q_base(t);
fim.fx(t,s)$(not fim0(s)) = q_base(t);
pfix.fx(t,s)$(not fix0(s)) = p_base(t);
pfim.fx(t,s)$(not fim0(s)) = p_base(t);
uc_loe_x.fx(t,s)$(not sum((r,ftrd),x0(r,s,ftrd))) = 1;
uc_loe_m.fx(t,s)$(not sum((r,ftrd),m0(r,s,ftrd))) = 1;
$if %loe%==0 row.fx(t) = 0;

* auxilary - additional income relative to numeraire
inc_extra.fx(t,r,h) = 0;

* allow for the option to perturb the starting values (useful for testing)
y.l(t,r,s) = y.l(t,r,s)+%perturb_start%;



* ------------------------------------------------------------------------------
* if this is a static run fix additional variables
* ------------------------------------------------------------------------------

if (card(t) eq 1,

* fix capital stock since its not used
  k.fx(t,r) = 1;

* fix terminal capital so that real investment is fixed
  kt.fx(r) = inv0(r)/k0(r);
  kht.fx(r,h) = inv0(r)/k0(r);

* price of capital has no meaning in the static model so fix it
  pk.fx(t,r) = 1;

* fix household investment
  invh.fx(t,r,h) = 1;
  prh.fx(t) = 1;

);



* ------------------------------------------------------------------------------
* define and policy changes
* ------------------------------------------------------------------------------

$if not %policy_file%==0 $include %policy_file%



* ------------------------------------------------------------------------------
* finalize abatement related parameters
* ------------------------------------------------------------------------------

* if all first period capital is extant then there is no new capital for
* abatement so make sure that there is no abatement requirement in that period
id_abate0(t,r,ss,s,v)$(tfirst(t) and extant_share(r,s) eq 1) = 0;
ld_abate0(t,r,s,v)$(tfirst(t) and extant_share(r,s) eq 1)    = 0;
kd_abate0(t,r,s,v)$(tfirst(t) and extant_share(r,s) eq 1)    = 0;

* aggregate abatement costs
abate0(t,r,s,v) = sum(ss, id_abate0(t,r,ss,s,v))
                  +(1+tp0(r))*ld_abate0(t,r,s,v)
                  +(1+tk0(r))*pr0*kd_abate0(t,r,s,v);

* define costs shares that are potentially conditional on abatement requirements
cs_abate(t,r,ss,s,v)   = (id_abate0(t,r,ss,s,v)             / abate0(t,r,s,v)                        )$(abate0(t,r,s,v));
cs_abate(t,r,"l",s,v)  = ((1+tp0(r))*ld_abate0(t,r,s,v)     / abate0(t,r,s,v)                        )$(abate0(t,r,s,v));
cs_abate(t,r,"k",s,v)  = ((1+tk0(r))*pr0*kd_abate0(t,r,s,v) / abate0(t,r,s,v)                        )$(abate0(t,r,s,v));
cs_y(t,r,"rklem",s)    = (rklem0(t,r,s)                     / (rklem0(t,r,s)+abate0(t,r,s,"new"))    )$(y0(r,s));
cs_y(t,r,"abate",s)    = (abate0(t,r,s,"new")               / (rklem0(t,r,s)+abate0(t,r,s,"new"))    )$(y0(r,s));
cs_y_ex(t,r,"l",s)     = ((1+tp0(r))*ld0(r,s)               / (rklem0(t,r,s)+abate0(t,r,s,"extant")) )$(extant_share(r,s) and y0(r,s));
cs_y_ex(t,r,"k",s)     = ((1+tk0(r))*pr0*kd0(r,s)           / (rklem0(t,r,s)+abate0(t,r,s,"extant")) )$(extant_share(r,s) and y0(r,s));
cs_y_ex(t,r,ss,s)      = (id0(r,ss,s)                       / (rklem0(t,r,s)+abate0(t,r,s,"extant")) )$(extant_share(r,s) and y0(r,s));
cs_y_ex(t,r,"res",s)   = ((1+tk0(r))*res0(r,s)              / (rklem0(t,r,s)+abate0(t,r,s,"extant")) )$(extant_share(r,s) and y0(r,s));
cs_y_ex(t,r,"abate",s) = (abate0(t,r,s,"extant")            / (rklem0(t,r,s)+abate0(t,r,s,"extant")) )$(extant_share(r,s) and y0(r,s));



* ------------------------------------------------------------------------------
* solve the model
* ------------------------------------------------------------------------------

* solve the model
solve sage using mcp;



* ------------------------------------------------------------------------------
* save the results
* ------------------------------------------------------------------------------

if (%gdx_save% eq 1, execute_unload "%gdx_results_file%");



* ------------------------------------------------------------------------------
* write output files with the results
* ------------------------------------------------------------------------------

* additional output variables
parameter
    gdp_real(t)         real gross dometic product
    tax_revenue(t,r)    total tax revenue
    income(t,r,h)       household income;

* real gdp
gdp_real(t) = sum(tt$tfirst(tt),
                    sum((r,s,h), pa.l(tt,r,s)*cd.l(t,r,s,h)*cd_base(t,r,s,h))
                   +sum((r,s), pa.l(tt,r,s)*i.l(t,r,s)*i0(r,s))
                   +sum((r,s), pa.l(tt,r,s)*g.l(t,r,s)*g_base(t,r,s))
                   +sum((r,s), px.l(tt,s)*x.l(t,r,s,"ftrd")*x0(r,s,"ftrd"))
                   -sum((r,s), pm.l(tt,s)*m.l(t,r,s,"ftrd")*m0(r,s,"ftrd")));

* total tax tax revenue
tax_revenue(t,r) =  sum(s, sum(h, tc(t,r,s)*pa.l(t,r,s)*cd.l(t,r,s,h)*cd_base(t,r,s,h))
                          +ty(t,r,s)*py.l(t,r,s)*y_ex.l(t,r,s)*y0(r,s)
                          +ty(t,r,s)*py.l(t,r,s)*y.l(t,r,s)*y0(r,s)
                          +tp(t,r)*pl.l(t,r)*ld_ex.l(t,r,s)*ld0(r,s)
                          +tp(t,r)*pl.l(t,r)*ld.l(t,r,s)*ld_base(t,r,s)
                          +tp(t,r)*pl.l(t,r)*sum(v, ld_abate.l(t,r,s,v)
                                                    *ld_abate0(t,r,s,v))
                          +tk(t,r)*pr_ex.l(t,r,s)*kd_ex.l(t,r,s)*kd0(r,s)
                          +tk(t,r)*pr.l(t,r)*kd.l(t,r,s)*kd_base(t,r,s)
                          +tk(t,r)*pr.l(t,r)*sum(v, kd_abate.l(t,r,s,v)
                                                   *kd_abate0(t,r,s,v))
                          +tk(t,r)*pres.l(t,r,s)*( res_ex.l(t,r,s)
                                                  +res.l(t,r,s))*res0(r,s))
                   +sum(h, tl_avg0(r,h)*pl.l(t,r)*l.l(t,r,h)*l0(r,h)
                          +tfica(t,r,h)*pl.l(t,r)*l.l(t,r,h)*l0(r,h));

* household income
income(t,r,h) = prh.l(t)*kh.l(t,r,h)*ke0(r,h)$(card(t) gt 1)
                +pr.l(t,r)*kh.l(t,r,h)*ke0(r,h)$(card(t) eq 1)
                +(1-tl(t,r,h)-tfica(t,r,h))*pl.l(t,r)*te(t,r,h)
                +(tl(t,r,h)-tl_avg0(r,h))*pl.l(t,r)*l.l(t,r,h)*l0(r,h)
                +prh_ex.l(t)*kh_ex.l(t,r,h)*ke0(r,h)
                +sum(s, pres.l(t,r,s)*(res_ex.l(t,r,s)+res.l(t,r,s))*rese0(r,s,h))
                +cpi.l(t)*oth_adj0(r,h)*oth_adj(t)
                +cpi.l(t)*incadj.l(t)*incadj_share(r,h)
                +cpi.l(t)*(tran_fed0(r,h)+tran_state0(r,h))*transfers(t)
                +cpi.l(t)*gint0*gint(t)*g0_share(r,h)
                -cpi.l(t)*deficit0*deficit(t)*g0_share(r,h)
                +pfx.l(t)*inc_row0*inc_row(t)*int0_share(r,h)
                +pfx.l(t)*tran_row0*tran_row(t)*int0_share(r,h)
                -pfx.l(t)*curactbal0*curactbal(t)*int0_share(r,h)
                +inc_extra.l(t,r,h);

* output csv results file
file results /"%output_file%"/;
results.pc = 5;
results.nd = 6;
put results;

put "parameter", "time", "region", "sector", "household", "sector.from", "value"/;

* relative prices
loop((t,r,s),   put   "py",       t.tl,  r.tl,  s.tl,  "",    "",    py.l(t,r,s)       / );
loop((t,r,s),   put   "pa",       t.tl,  r.tl,  s.tl,  "",    "",    pa.l(t,r,s)       / );
loop((t,r,h),   put   "pc",       t.tl,  r.tl,  "",    h.tl,  "",    pc.l(t,r,h)       / );
loop((t,r,h),   put   "pcl",      t.tl,  r.tl,  "",    h.tl,  "",    pcl.l(t,r,h)      / );
loop((t,r),     put   "pl",       t.tl,  r.tl,  "",    "",    "",    pl.l(t,r)         / );
loop((t,r,s),   put   "pr_ex",    t.tl,  r.tl,  s.tl,  "",    "",    pr_ex.l(t,r,s)    / );
loop((t,r),     put   "pr_ex_agg",t.tl,  r.tl,  "",    "",    "",    pr_ex_agg.l(t,r)  / );
loop((t),       put   "prh_ex",   t.tl,  "",    "",    "",    "",    prh_ex.l(t)       / );
loop((t,r,s),   put   "pres",     t.tl,  r.tl,  s.tl,  "",    "",    pres.l(t,r,s)     / );
loop((t,r),     put   "pr",       t.tl,  r.tl,  "",    "",    "",    pr.l(t,r)         / );
loop((t,r),     put   "pk",       t.tl,  r.tl,  "",    "",    "",    pk.l(t,r)         / );
loop((t,r),     put   "pgov",     t.tl,  r.tl,  "",    "",    "",    pgov.l(t,r)       / );
loop((t,s),     put   "pn",       t.tl,  "",    s.tl,  "",    "",    pn.l(t,s)         / );
loop((t,r,s),   put   "pd",       t.tl,  r.tl,  s.tl,  "",    "",    pd.l(t,r,s)       / );
loop((t),       put   "pfx",      t.tl,  "",    "",    "",    "",    pfx.l(t)          / );
loop((t,r,s),   put   "px",       t.tl,  "",    s.tl,  "",    "",    px.l(t,s)         / );
loop((t,r,s),   put   "pm",       t.tl,  "",    s.tl,  "",    "",    pm.l(t,s)         / );
loop((r),       put   "pkt",      "",    r.tl,  "",    "",    "",    pkt.l(r)          / );
loop((t),       put   "cpi",      t.tl,  "",    "",    "",    "",    cpi.l(t)          / );
loop((t),       put   "prh",      t.tl,  "",    "",    "",    "",    prh.l(t)          / );

* quantities
loop((t,r,s),   put   "y_ex",     t.tl,  r.tl,  s.tl,  "",     "",    (y_ex.l(t,r,s)*y0(r,s))            / );
loop((t,r,s),   put   "y",        t.tl,  r.tl,  s.tl,  "",     "",    (y.l(t,r,s)*y0(r,s))               / );
loop((t,r,s),   put   "a",        t.tl,  r.tl,  s.tl,  "",     "",    (a.l(t,r,s)*a0(r,s))               / );
loop((t,r,s),   put   "d",        t.tl,  r.tl,  s.tl,  "",     "",    (d.l(t,r,s)*d0(r,s))               / );
loop((t,r,s),   put   "i",        t.tl,  r.tl,  s.tl,  "",     "",    (i.l(t,r,s)*i0(r,s))               / );
loop((t,r,s,h), put   "cd",       t.tl,  r.tl,  s.tl,  h.tl,   "",    (cd.l(t,r,s,h)*cd_base(t,r,s,h))   / );
loop((t,r,s,h), put   "dcd",      t.tl,  r.tl,  s.tl,  h.tl,   "",    (dcd.l(t,r,s,h)*dcd_base(t,r,s,h)) / );
loop((t,r,s,h), put   "scd",      t.tl,  r.tl,  s.tl,  h.tl,   "",    (scd.l(t,r,s,h)*scd_base(t,r,s,h)) / );
loop((t,r,h),   put   "c",        t.tl,  r.tl,  "",    h.tl,   "",    (c.l(t,r,h)*c0(r,h))               / );
loop((t,r,h),   put   "dc",       t.tl,  r.tl,  "",    h.tl,   "",    (dc.l(t,r,h)*dc0(r,h))             / );
loop((t,r,h),   put   "cl",       t.tl,  r.tl,  "",    h.tl,   "",    (cl.l(t,r,h)*cl0(r,h))             / );
loop((t,r,s),   put   "g",        t.tl,  r.tl,  s.tl,  "",     "",    (g.l(t,r,s)*g_base(t,r,s))         / );
loop((t,r,s),   put   "x_f",      t.tl,  r.tl,  s.tl,  "",     "",    (x.l(t,r,s,"ftrd")*x0(r,s,"ftrd")) / );
loop((t,r,s),   put   "m_f",      t.tl,  r.tl,  s.tl,  "",     "",    (m.l(t,r,s,"ftrd")*m0(r,s,"ftrd")) / );
loop((t,r,s),   put   "x_d",      t.tl,  r.tl,  s.tl,  "",     "",    (x.l(t,r,s,"dtrd")*x0(r,s,"dtrd")) / );
loop((t,r,s),   put   "m_d",      t.tl,  r.tl,  s.tl,  "",     "",    (m.l(t,r,s,"dtrd")*m0(r,s,"dtrd")) / );
loop((t,r,s),   put   "ld_ex",    t.tl,  r.tl,  s.tl,  "",     "",    (ld_ex.l(t,r,s)*ld0(r,s))          / );
loop((t,r,s),   put   "ld",       t.tl,  r.tl,  s.tl,  "",     "",    (ld.l(t,r,s)*ld_base(t,r,s))       / );
loop((t,r,h),   put   "leisure",  t.tl,  r.tl,  "",    h.tl,   "",    (leis.l(t,r,h)*leis0(r,h))         / );
loop((t,r,h),   put   "l",        t.tl,  r.tl,  "",    h.tl,   "",    (l.l(t,r,h)*l0(r,h))               / );
loop((t,r,s),   put   "kd_ex",    t.tl,  r.tl,  s.tl,  "",     "",    (kd_ex.l(t,r,s)*kd0(r,s))          / );
loop((t,r,s),   put   "kd",       t.tl,  r.tl,  s.tl,  "",     "",    (kd.l(t,r,s)*kd_base(t,r,s))       / );
loop((t,r,h),   put   "r",        t.tl,  r.tl,  "",    h.tl,   "",    (kh.l(t,r,h)*ke0(r,h))             / );
loop((t,r,h),   put   "r_ex",     t.tl,  r.tl,  "",    h.tl,   "",    (k_ex.l(t,r)*ke0(r,h))             / );
loop((t,r,s),   put   "res_ex",   t.tl,  r.tl,  s.tl,  "",     "",    (res_ex.l(t,r,s)*res0(r,s))        / );
loop((t,r,s),   put   "res",      t.tl,  r.tl,  s.tl,  "",     "",    (res.l(t,r,s)*res0(r,s))           / );
loop((t,r,s,ss),put   "id",       t.tl,  r.tl,  s.tl,  "",   ss.tl,   (id.l(t,r,ss,s)*id_base(t,r,ss,s)) / );
loop((t,r,s,ss),put   "id_ex",    t.tl,  r.tl,  s.tl,  "",   ss.tl,   (id_ex.l(t,r,ss,s)*id0(r,ss,s))    / );

loop((t,r),     put   "k",        t.tl,  r.tl,  "",    "",     "",    (k.l(t,r)*k0(r))                   / );
loop((t,r),     put   "k_ex",     t.tl,  r.tl,  "",    "",     "",    (k_ex.l(t,r)*sum(h, ke0(r,h)))     / );
loop((r),       put   "kt",       "",    r.tl,  "",    "",    "",     (kt.l(r)*k0(r))                    / );
loop((t,r,h),   put   "invh",     t.tl,  r.tl,  "",    h.tl,   "",    (invh.l(t,r,h)*invh0(r,h))         / );
loop((t,r,h),   put   "kh",       t.tl,  r.tl,  "",    h.tl,   "",    (kh.l(t,r,h)*ke0(r,h))             / );
loop((t,r,h),   put   "kh_ex",    t.tl,  r.tl,  "",    h.tl,   "",    (kh_ex.l(t,r,h)*ke0(r,h))          / );

* household
loop((r,h),     put   "w",       "",    r.tl,  "",    h.tl,   "",      w.l(r,h)            / );
loop((t,r,h),   put   "te",       t.tl,  r.tl,  "",    h.tl,   "",    te(t,r,h)                  / );
loop((t,r,h),   put   "n",        t.tl,  r.tl,  "",    h.tl,   "",    n(t,r,h)                   / );

* abatement inputs
loop((t,r,s),   put   "ld_abate",    t.tl,  r.tl,  s.tl,  "",     "",    (ld_abate.l(t,r,s,"new")*ld_abate0(t,r,s,"new"))             / );
loop((t,r,s),   put   "ld_abate_ex", t.tl,  r.tl,  s.tl,  "",     "",    (ld_abate.l(t,r,s,"extant")*ld_abate0(t,r,s,"extant"))       / );
loop((t,r,s),   put   "kd_abate",    t.tl,  r.tl,  s.tl,  "",     "",    (kd_abate.l(t,r,s,"new")*kd_abate0(t,r,s,"new"))             / );
loop((t,r,s),   put   "kd_abate_ex", t.tl,  r.tl,  s.tl,  "",     "",    (kd_abate.l(t,r,s,"extant")*kd_abate0(t,r,s,"extant"))       / );
loop((t,r,s,ss),put   "id_abate",    t.tl,  r.tl,  s.tl,  "",   ss.tl,   (id_abate.l(t,r,ss,s,"new")*id_abate0(t,r,ss,s,"new"))       / );
loop((t,r,s,ss),put   "id_abate_ex", t.tl,  r.tl,  s.tl,  "",   ss.tl,   (id_abate.l(t,r,ss,s,"extant")*id_abate0(t,r,ss,s,"extant")) / );

* macroeconomic variables
loop((t),       put   "gdp",      t.tl,  "",    "",    "",     "",    (gdp.l(t)*gdp0)            / );
loop((t),       put   "gdp_real", t.tl,  "",    "",    "",     "",    gdp_real(t)                / );

* government accounts
loop((t),       put   "deficit",  t.tl   "",    "",    "",     "",    (deficit(t)*deficit0)      / );
loop((t),       put   "debt",     t.tl   "",    "",    "",     "",    (debt(t)*debt0)            / );
loop((t),       put   "gint",     t.tl   "",    "",    "",     "",    (gint(t)*gint0)            / );
loop((t),       put   "gint_row", t.tl   "",    "",    "",     "",    (gint_row(t)*gint_row0)    / );
loop((t,r,h),   put   "incadj",   t.tl,  r.tl,  "",    h.tl,   "",    (incadj.l(t)*incadj_share(r,h)) / );
loop((t,r,h),   put   "oth_adj",  t.tl,  r.tl,  "",    h.tl,   "",    (oth_adj(t)*oth_adj0(r,h)) / );
loop((t,r,h),   put   "transfers",t.tl,  r.tl,  "",    h.tl,   "",    (transfers(t)*(tran_fed0(r,h)+tran_state0(r,h))) / );
loop((t,r,h),   put   "income",   t.tl,  r.tl,  "",    h.tl,   "",    income(t,r,h)              / );

* foreign accounts
loop((t),       put   "curactbal",t.tl   "",    "",    "",     "",    (curactbal(t)*curactbal0)  / );
loop((t),       put   "tran_row", t.tl   "",    "",    "",     "",    (tran_row(t)*tran_row0)    / );
loop((t),       put   "inc_row",  t.tl   "",    "",    "",     "",    (inc_row(t)*inc_row0)      / );

* calibrated parameters
loop((t,r,s),   put   "se_rklem", t.tl,  r.tl,  s.tl,     "",    "",    se_rklem(t,r,s)   / );
loop((r,h),     put   "se_cl",      "",  r.tl,    "",   h.tl,    "",    se_cl(r,h)        / );

* taxes
loop((t,r),     put   "tk",          t.tl,  r.tl,  "",    "",    "",     tk(t,r)          / );
loop((t,r),     put   "tp",          t.tl,  r.tl,  "",    "",    "",     tp(t,r)          / );
loop((t,r,h),   put   "tl",          t.tl,  r.tl,  "",  h.tl,    "",     tl(t,r,h)        / );
loop((t,r,h),   put   "tfica",       t.tl,  r.tl,  "",  h.tl,    "",     tfica(t,r,h)     / );
loop((t,r,s),   put   "tc",          t.tl,  r.tl,  s.tl,  "",    "",     tc(t,r,s)        / );
loop((t,r,s),   put   "ty",          t.tl,  r.tl,  s.tl,  "",    "",     ty(t,r,s)        / );
loop((t,r),     put   "tax_revenue", t.tl,  r.tl,  "",    "",    "",     tax_revenue(t,r) / );
loop((r,h),     put   "tl_avg",        "",  r.tl,  "",  h.tl,    "",     tl_avg0(r,h)     / );

putclose;



* ------------------------------------------------------------------------------
* display additional statistics for the model run
* ------------------------------------------------------------------------------

parameters
  inv0_end(r)           endogenous benchmark investment
  c0_end(r,h)           endogenous benchmark consumption;

inv0_end(r) = sum((t,s)$tfirst(t), pa.l(t,r,s)*i.l(t,r,s)*i0(r,s));
display inv0, inv0_end;

c0_end(r,h) = sum((t,s)$tfirst(t), (1+tc(t,r,s))*pa.l(t,r,s)*cd.l(t,r,s,h)
                                                          *cd_base(t,r,s,h));
display c0, c0_end;

* calculate the implicit annual interest rate in each region
parameters
  r_annual(r,h)         calibrated annual interest rate;
r_annual(r,h) = r0(r)/k0(r)-delta;
display r_annual, rho, k0;

* display gdp growth
parameters
  gdp_growth(t)         endogenous gdp growth;
gdp_growth(t)$(ord(t) gt 1) = (gdp_real(t)/gdp_real(t-1))**(1/interval(t-1))-1;
display gdp_growth;



* ------------------------------------------------------------------------------
* run solution checks
* ------------------------------------------------------------------------------

parameters
  zero_tol         tolerance for assuming a zero difference        / 1e-4 /;



* ------------------------------------------------------------------------------
* gdp calculation
* ------------------------------------------------------------------------------

* Calculation of nominal GDP from the expenditure and value added sides
parameters
  gdp_e(t)           nominal gdp - calculated based on expenditures
  gdp_va(t)          nominal gdp - calculated based on value added;

* nominal gdp - calculated based on expenditures
gdp_e(t)  = sum(r, sum((h,s), pa.l(t,r,s)*cd.l(t,r,s,h)*cd_base(t,r,s,h))
            +sum(s, pa.l(t,r,s)*(g.l(t,r,s)*g_base(t,r,s)+i.l(t,r,s)*i0(r,s)))
            +sum(s, px.l(t,s)*sum(ftrd, x0(r,s,ftrd)*x.l(t,r,s,ftrd)))
            -sum(s, pm.l(t,s)*sum(ftrd, m0(r,s,ftrd)*m.l(t,r,s,ftrd))));

* nominal gdp - calculated based on value added
gdp_va(t) = sum(r, sum(s, pl.l(t,r)*ld_ex.l(t,r,s)*ld0(r,s)
                         +pl.l(t,r)*ld.l(t,r,s)*ld_base(t,r,s)
                         +pl.l(t,r)*sum(v, ld_abate.l(t,r,s,v)
                                          *ld_abate0(t,r,s,v))
                         +pr_ex.l(t,r,s)*kd_ex.l(t,r,s)*kd0(r,s)
                         +pr.l(t,r)*kd.l(t,r,s)*kd_base(t,r,s)
                         +pr.l(t,r)*sum(v, kd_abate.l(t,r,s,v)
                                          *kd_abate0(t,r,s,v))
                         +pres.l(t,r,s)*(res_ex.l(t,r,s)
                                         +res.l(t,r,s))*res0(r,s))
                   +sum(s, tp(t,r)*pl.l(t,r)*ld_ex.l(t,r,s)*ld0(r,s)
                          +tp(t,r)*pl.l(t,r)*ld.l(t,r,s)*ld_base(t,r,s)
                          +tp(t,r)*pl.l(t,r)*sum(v, ld_abate.l(t,r,s,v)
                                                   *ld_abate0(t,r,s,v))
                          +tk(t,r)*pr_ex.l(t,r,s)*kd_ex.l(t,r,s)*kd0(r,s)
                          +tk(t,r)*pr.l(t,r)*kd.l(t,r,s)*kd_base(t,r,s)
                          +tk(t,r)*pr.l(t,r)*sum(v, kd_abate.l(t,r,s,v)
                                                   *kd_abate0(t,r,s,v))
                          +tk(t,r)*pres.l(t,r,s)*(res_ex.l(t,r,s)
                                                  +res.l(t,r,s))*res0(r,s)
                          +ty(t,r,s)*py.l(t,r,s)*y_ex.l(t,r,s)*y0(r,s)
                          +ty(t,r,s)*py.l(t,r,s)*y.l(t,r,s)*y0(r,s)));

display gdp_e, gdp_va;

* savings rate
parameters
  savings_rate(t);
savings_rate(t) = sum((r,s), pa.l(t,r,s)*i.l(t,r,s)*i0(r,s))/gdp_e(t);
display savings_rate;



* ------------------------------------------------------------------------------
* sam - build the social accounting matrix for each region plus national
* ------------------------------------------------------------------------------

* creates a micro sam by year and region based on the post solve equilbrium. the
* sam is then aggregated to the national level for each year and the row and
* column totals are compared to check whether the post solve sam is balanced.
* note that the regional sams do not balance because investments in new capital
* can move across regions.
*
* accounts in sam:
*  s        sectors
*  lab      labor
*  cap      new capital
*  cap_ex   extant capital
*  nrs      natural resources
*  h        households
*  gov      government
*  inv      investment
*  ty       production tax
*  tp       payroll tax (not active)
*  tk       capital income tax
*  tl       labor income tax
*  tfica    fica tax
*  tc       consumption tax
*  dtrd     national trade
*  ftrd     foreign trade

sets
  accounts(*)           accounts in the sam;

parameters
  sam(t,r,*,*)          micro sam by region and year
  sam_national(t,*,*)   sam aggregated to the national level by year
  row_totals(t,*)       national sam row totals by year
  col_totals(t,*)       national sam column totals by year
  row_col_diff(t,*)     row and column comparison by year in national sam;

* define the accounts in the sam
accounts(s)        = yes;
accounts("lab")    = yes;
accounts("cap")    = yes;
accounts("cap_ex") = yes;
accounts("nrs")    = yes;
accounts(h)        = yes;
accounts("gov")    = yes;
accounts("inv")    = yes;
accounts("ty")     = yes;
accounts("tp")     = yes;
accounts("tk")     = yes;
accounts("tl")     = yes;
accounts("tfica")  = yes;
accounts("tc")     = yes;
accounts("dtrd")   = yes;
accounts("ftrd")   = yes;
alias (accounts,aa);

* initialize all account entries to zero
sam(t,r,accounts,aa) = 0;

* intermediate commodity use in production activities
sam(t,r,ss,s) = pa.l(t,r,ss)*( id.l(t,r,ss,s)*id_base(t,r,ss,s)
                              +id_ex.l(t,r,ss,s)*id0(r,ss,s)
                              +sum(v, id_abate.l(t,r,ss,s,v)
                                     *id_abate0(t,r,ss,s,v)));

* value-added use in production activities
sam(t,r,"lab",s) = pl.l(t,r)*( ld.l(t,r,s)*ld_base(t,r,s)
                              +ld_ex.l(t,r,s)*ld0(r,s)
                              +sum(v, ld_abate.l(t,r,s,v)*ld_abate0(t,r,s,v)));
sam(t,r,"cap",s) = pr.l(t,r)*( kd.l(t,r,s)*kd_base(t,r,s)
                              +sum(v, kd_abate.l(t,r,s,v)*kd_abate0(t,r,s,v)));
sam(t,r,"cap_ex",s) = pr_ex.l(t,r,s)*kd_ex.l(t,r,s)*kd0(r,s);
sam(t,r,"nrs",s) = pres.l(t,r,s)*(res.l(t,r,s)+res_ex.l(t,r,s))*res0(r,s);

* taxes paid by production activities
sam(t,r,"ty",s) = ty(t,r,s)*py.l(t,r,s)*(y.l(t,r,s)+y_ex.l(t,r,s))*y0(r,s);
sam(t,r,"tp",s) = tp(t,r)*sam(t,r,"lab",s);
sam(t,r,"tk",s) = tk(t,r)*( sam(t,r,"cap",s)+sam(t,r,"cap_ex",s)
                           +sam(t,r,"nrs",s));

* commodity imports
sam(t,r,"dtrd",s) = pn.l(t,s)*m.l(t,r,s,"dtrd")*m0(r,s,"dtrd");
sam(t,r,"ftrd",s) = pm.l(t,s)*m.l(t,r,s,"ftrd")*m0(r,s,"ftrd");

* final commodity demands
sam(t,r,s,h) = pa.l(t,r,s)*cd.l(t,r,s,h)*cd_base(t,r,s,h);
sam(t,r,s,"gov") = pa.l(t,r,s)*g.l(t,r,s)*g_base(t,r,s);
sam(t,r,s,"inv") = pa.l(t,r,s)*i.l(t,r,s)*i0(r,s);

* commodity exports
sam(t,r,s,"dtrd") = pn.l(t,s)*x.l(t,r,s,"dtrd")*x0(r,s,"dtrd");
sam(t,r,s,"ftrd") = px.l(t,s)*x.l(t,r,s,"ftrd")*x0(r,s,"ftrd");

* value-added payments to households excluding new capital
sam(t,r,h,"cap") = prh.l(t)*kh.l(t,r,h)*ke0(r,h);
sam(t,r,h,"lab") = pl.l(t,r)*l.l(t,r,h)*l0(r,h);
sam(t,r,h,"cap_ex") = prh_ex.l(t)*kh_ex.l(t,r,h)*ke0(r,h);
sam(t,r,h,"nrs") = sum(s, pres.l(t,r,s)*(res.l(t,r,s)+res_ex.l(t,r,s))
                         *rese0(r,s,h));

* government transfers to households
sam(t,r,h,"gov") = cpi.l(t)*((tran_fed0(r,h)+tran_state0(r,h))*transfers(t)
                             +gint0*gint(t)*g0_share(r,h)
                             +incadj.l(t)*incadj_share(r,h)
                             +oth_adj(t)*oth_adj0(r,h));

* household tax payments
sam(t,r,"tl",h) = tl_avg0(r,h)*sam(t,r,h,"lab");
sam(t,r,"tfica",h) = tfica(t,r,h)*sam(t,r,h,"lab");
sam(t,r,"tc",h) = sum(s, tc(t,r,s)*sam(t,r,s,h));

* government deficit
sam(t,r,"gov",h) = cpi.l(t)*deficit0*deficit(t)*g0_share(r,h);

* household savings - "investment" from households in new capital account
sam(t,r,"cap",h) = (pkh.l(t+1)+pkht.l$tlast(t))*invh.l(t,r,h)*invh0(r,h);

* investment in new capital - from new capital account to investment account
sam(t,r,"inv","cap") = uc_inv.l(t,r)*inv.l(t,r)*inv0(r);

* government tax revenue
sam(t,r,"gov","ty") = sum(s, sam(t,r,"ty",s));
sam(t,r,"gov","tp") = sum(s, sam(t,r,"tp",s));
sam(t,r,"gov","tk") = sum(s, sam(t,r,"tk",s));
sam(t,r,"gov","tl") = sum(h, sam(t,r,"tl",h));
sam(t,r,"gov","tfica") = sum(h, sam(t,r,"tfica",h));
sam(t,r,"gov","tc") = sum(h, sam(t,r,"tc",h));

* balance of payments
sam(t,r,h,"ftrd") = pfx.l(t)*inc_row0*inc_row(t)*int0_share(r,h)
                    +pfx.l(t)*tran_row0*tran_row(t)*int0_share(r,h)
                    -pfx.l(t)*curactbal0*curactbal(t)*int0_share(r,h);

* government interest payments to row
sam(t,r,"ftrd","gov") = pfx.l(t)*gint_row0*gint_row(t)
                        *tax_revenue(t,r)/sum(rr, tax_revenue(t,rr));

* aggregate accross regions to form the national sam
sam_national(t,accounts,aa) = 0;
sam_national(t,accounts,aa) = sum(r, sam(t,r,accounts,aa));

* compare row and column totals by year in the national sam
row_totals(t,accounts) = sum(aa, sam_national(t,accounts,aa));
col_totals(t,accounts) = sum(aa, sam_national(t,aa,accounts));
row_col_diff(t,accounts) = row_totals(t,accounts)-col_totals(t,accounts);

display sam_national, row_totals, col_totals, row_col_diff;



* ------------------------------------------------------------------------------
* check for discrepency between household and regional prices of capital
* ------------------------------------------------------------------------------

parameters
  pkh_check(t)          difference between endogenous and expected household capital price;

* difference between endogenous and expected household capital price
pkh_check(t)$(sum(r, k.l(t,r)*k0(r))) = pkh.l(t)
                                        -sum(r, pk.l(t,r)*k.l(t,r)*k0(r))
                                           /sum(r, k.l(t,r)*k0(r));
display pkh_check;



* ------------------------------------------------------------------------------
* output results of the solution check to the log file
* ------------------------------------------------------------------------------

parameters
  fail_count            number of checks that failed;

files
  log                   connection to the gams listing file        /''/;

put log;

put "----------------------------------------------------" /;
put "Solution Check Results" /;
put "----------------------------------------------------" /;
fail_count = 0;
loop(t,
  loop(accounts,
    if (abs(row_col_diff(t,accounts))>zero_tol,
      put t.tl, " ", accounts.tl, " account check       FAILED" /;
      fail_count = fail_count+1;
    );
  );
);
loop(t,
  if (abs(gdp_e(t)-gdp_va(t))>zero_tol,
    put t.tl, " GDP          check               FAILED" /;
    fail_count = fail_count+1;
  );
);
loop(t,
  if (abs(pkh_check(t))>zero_tol,
    put t.tl, " capital      account check       FAILED" /;
    fail_count = fail_count+1;
  );
);
if (fail_count=0,
  put "all checks passed" /;
);
put "----------------------------------------------------" /;
putclose;


* if there is no policy file or baseline file then there is no cv to calculate
$if %policy_file%==0 $goto epilogue_start
$if %gdx_baseline_file%==0 $goto epilogue_start




* ------------------------------------------------------------------------------
* solve for equivalent variation
* ------------------------------------------------------------------------------

* solves for the willingness to pay for to forgo the policy change measured as
* the change in expenditures required to achieve the policy case welfare level
* at the baseline prices

parameters
  w_policy(r,h)         welfare in policy simulation
  exp_diff(t,r,h)       expenditure difference between baseline and ev case
  ev(r,h)               equivalent variation - finite horizon
  ev_annual(r,h)        equivalent variation - annual step and finite horizon
  ev_inf(r,h)           equivalent variation - annual step and infinite horizon
  c_ev(t,r,h)           equivalent variation consumption path
  leis_ev(t,r,h)        equivalent variation leisure path
  step_multiplier(t)    multiplier for annual linear interpolation
  exp_base(t,r,h)       expenditures in the baseline
  cl_base(r,h)          full consumption - present value
  cl_base_annual(r,h)   full consumption - annual step and finite horizon
  cl_base_inf(r,h)      full consumption - annual step and infinite horizon;

variables
  lambda_t(r,h)         terminal shadow price on consumption

equations
  aux_ev(r,h)           auxilary - equivalent variation constraint
  aux_lambda(t,r,h)     auxilary - lambda definition based on terminal value;

* save policy welfare to be used later in solving for ev
w_policy(r,h) = w.l(r,h);

* get the baseline prices
execute_load "%gdx_baseline_file%" pa, pk, pr, pl, pfx, pkt, kt, pr_ex, incadj,
                                   pres, res_ex, res, cpi, prh_ex, tc, tl,
                                   tfica;

* fix the prices at the baseline levels
pa.fx(t,r,s)      = pa.l(t,r,s);
pk.fx(t,r)        = pk.l(t,r);
pr.fx(t,r)        = pr.l(t,r);
pl.fx(t,r)        = pl.l(t,r);
pfx.fx(t)         = pfx.l(t);
pkt.fx(r)         = pkt.l(r);
kt.fx(r)          = kt.l(r);
pr_ex.fx(t,r,s)   = pr_ex.l(t,r,s);
incadj.fx(t)      = incadj.l(t);
pres.fx(t,r,s)    = pres.l(t,r,s);
res_ex.fx(t,r,s)  = res_ex.l(t,r,s);
res.fx(t,r,s)     = res.l(t,r,s);
cpi.fx(t)         = cpi.l(t);
prh_ex.fx(t)      = prh_ex.l(t);

* constraint for ev calculation is that welfare must equal policy level
aux_ev(r,h)..
  w_policy(r,h) =e= w(r,h);

* calculate lambda based on the terminal value that equates welfare
aux_lambda(t,r,h)..
  lambda(t,r,h) =e= (1/(1+rho(r,h)))**(sum(tlast, tlast.val)-t.val)
                    *lambda_t(r,h);

* define the model for computing ev
model calc_ev /
  zp_c.dc
  zp_cl.pcl

  hde_dc.pc
  hde_c.c
  hde_cd.cd
  hde_dcd.dcd
  hde_scd.scd
  hde_leis.leis

  foc_cl.cl

  uce_c.uc_c
  uce_cl.uc_cl

  aux_w.w
  aux_ev.lambda_t
  aux_lambda.lambda
/;

* iteration limit
calc_ev.iterlim = 100000;

* use model scaling
calc_ev.scaleopt = 1;

* use the path options file
calc_ev.optfile = 1;

* solve the mcp problem
solve calc_ev using mcp;

* save the equivalent variation consumption and leisure paths
c_ev(t,r,h) = dc.l(t,r,h);
leis_ev(t,r,h) = leis.l(t,r,h);

* get the baseline consumption and leisure paths
execute_load "%gdx_baseline_file%" dc, leis;

* calculate the present value reduction in expenditures required to reach
* the policy case welfare level at the baseline prices
exp_diff(t,r,h) = (c_ev(t,r,h)-dc.l(t,r,h))*dc0(r,h)*pc.l(t,r,h)
                  +(leis_ev(t,r,h)-leis.l(t,r,h))*leis0(r,h)
                    *(1-tl(t,r,h)-tfica(t,r,h))*pl.l(t,r);

* calculate ev as the sum of present value reduction in expenditures
ev(r,h) = sum(t, exp_diff(t,r,h));

* cumulative sum -> sum_{i=0}^{interval}{i-1} used in the linear
* interpolation to annual time steps
set counter / 1*100 /;
step_multiplier(t) = sum(counter$(counter.val le interval(t)), counter.val-1);

* calculate ev for the model's finite time horizon but at an annual time step by
* linearly interpolating consumption and leisure paths between simulated years
ev_annual(r,h) = sum(t$(ord(t) ne card(t)),
                       interval(t)*exp_diff(t,r,h)
                       +step_multiplier(t)*(exp_diff(t+1,r,h)
                                            -exp_diff(t,r,h))/interval(t))
                 +sum(t$(ord(t) eq card(t)), exp_diff(t,r,h));

* calculate equivalent variation for an infinite time horizon assuming
* that the economy is on a balanced growth path
ev_inf(r,h) = ev_annual(r,h)
              +sum(t$tlast(t), exp_diff(t,r,h))
                *(1/(1-(sum(tlast, q_base(tlast))
                          /sum(t$(ord(t) eq card(t)-1), q_base(t))
                       )**(1/sum(tlast, interval(tlast)))
                       /(1+rbar))-1)$(card(t) gt 1);

* get the baseline consumption and leisure prices
execute_load "%gdx_baseline_file%" pa, cd, tc, pl;

* baseline full consumption (consumption and leisure)
exp_base(t,r,h) = sum(s, (1+tc(t,r,s))*cd.l(t,r,s,h)*cd_base(t,r,s,h)*pa.l(t,r,s))
                  +leis.l(t,r,h)*leis0(r,h)*(1-tl(t,r,h)-tfica(t,r,h))
                    *pl.l(t,r);

* present value of baseline full consumption (consumption and leisure)
cl_base(r,h) = sum(t, exp_base(t,r,h));

* calculate baseline full consumption for the model's finite time horizon but at
* an annual time step by linearly interpolating consumption and leisure paths
* between simulated years
cl_base_annual(r,h) = sum(t$(ord(t) ne card(t)),
                            interval(t)*exp_base(t,r,h)
                            +step_multiplier(t)*(exp_base(t+1,r,h)
                                                 -exp_base(t,r,h))/interval(t))
                      +sum(t$(ord(t) eq card(t)), exp_base(t,r,h));

* calculate baseline full consumption for an infinite time horizon assuming
* that the economy is on a balanced growth path
cl_base_inf(r,h) = cl_base_annual(r,h)
                   +sum(t$tlast(t), exp_base(t,r,h))
                     *(1/(1-(sum(tlast, q_base(tlast))
                               /sum(t$(ord(t) eq card(t)-1), q_base(t))
                            )**(1/sum(tlast, interval(tlast)))
                            /(1+rbar))-1)$(card(t) gt 1);

* output ev to results file
results.ap = 1;
put results;
loop((t,r,h), put "exp_diff",  t.tl, r.tl, "", h.tl, "", exp_diff(t,r,h) / );
loop((r,h),   put "ev",          "", r.tl, "", h.tl, "", ev(r,h)         / );
loop((r,h),   put "ev_annual",   "", r.tl, "", h.tl, "", ev_annual(r,h)  / );
loop((r,h),   put "ev_inf",      "", r.tl, "", h.tl, "", ev_inf(r,h)     / );

loop((t,r,h), put "exp_base",       t.tl, r.tl, "", h.tl, "", exp_base(t,r,h)     / );
loop((r,h),   put "cl_base",          "", r.tl, "", h.tl, "", cl_base(r,h)        / );
loop((r,h),   put "cl_base_annual",   "", r.tl, "", h.tl, "", cl_base_annual(r,h) / );
loop((r,h),   put "cl_base_inf",      "", r.tl, "", h.tl, "", cl_base_inf(r,h)    / );
putclose;



* ------------------------------------------------------------------------------
* run epilogue code if provided
* ------------------------------------------------------------------------------

$label epilogue_start

$if %epilogue%==0 $exit
$include %epilogue%
