# download_file.R
#
# downloads a file if not present locally and extracts the contents if it is a
# compressed archive
#
# inputs:
#   url: url where the remote file resides
#   file.name: name of the remote file
#   dir: local directory to save the file [defulat is "."]
#   check.file: optional name of local file, where if present the download will
#               not occur. useful if the remote file is a compressed archive and
#               it shouldn't be downloaded if its contents is already present
#

download_file = function(url,file.name,dir=".",check.file=NA) {

  library(httr)

  # create the directory if needed
  if (!dir.exists(dir))
    dir.create(dir)

  # if no check file name is provided use the remote file name
  if (is.na(check.file))
    check.file = file.name

  # if the check file is present then do nothing further
  if (file.exists(file.path(dir,check.file)))
    return()

  # download the file
  cat(paste0("downloading ",file.name,"... \n"))
  GET(paste0(url,file.name),
      write_disk(file.path(dir,file.name),overwrite=TRUE),
      progress())

  # if the file is a zip archive extract the contents and delete the archive
  if (tools::file_ext(file.name)=="zip") {
    cat("extracting zip file...\n")
    unzip(file.path(dir,file.name),exdir=dir)
    file.remove(file.path(dir,file.name))
  }

  cat("\n")

}
